/* $Id: SaleItemDetailImpl.java 710 2009-09-19 13:54:04Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk;

import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail;

/** Represent sale item detail for displaing GUI.  
  * 
  * @author Ondrej Cerny
  *
  */
public class SaleItemDetailImpl implements SaleItemDetail {
	/** Generated serial version UID. */
	private static final long serialVersionUID = -259323170749972378L;
	
	// data fields:
	/** Stock item identification. */
	private String EAN;
	/** Product name. */
	private String name;
	/** Product description. */
	private String desc;
	/** Sale amount. */
	private double saleAmount;
	/** Stock amount. */
	private double stockAmount;
	/** Unit price. */
	private double unitPrice;
	/** Total price for this sale item. */
	private double total;

	/** Creates new sale item detail.
	  * 
	  * @param EAN unique stock item identification
	  * @param name product name
	  * @param desc product description
	  * @param saleAmount sale amount
	  * @param stockAmount stock amount
	  * @param unitPrice unit price
	  * @param totalPrice total price of this sale itme
	  */
	public SaleItemDetailImpl(
			String EAN,
			String name,
			String desc,
			double saleAmount,
			double stockAmount,
			double unitPrice,
			double totalPrice) {
		this.EAN = EAN;
		this.name = name;
		this.desc = desc;
		this.saleAmount = saleAmount;
		this.stockAmount = stockAmount;
		this.unitPrice = unitPrice;
		this.total = totalPrice;
	}

	/** Creates new sale item detail with computed total price.
	  * 
	  * <p>
	  * Sets <code>total</code> to <code>unitPrice</code> multiplied by
	  * <code>saleAmount</code>.
	  * </p>
	  * 
	  * @param EAN unique stock item identification
	  * @param name product name
	  * @param desc product description
	  * @param saleAmount sale amount
	  * @param stockAmount stock amount
	  * @param unitPrice unit price
	  * @param totalPrice total price of this sale itme
	  */
	public SaleItemDetailImpl(
			String EAN,
			String name,
			String desc,
			double saleAmount,
			double stockAmount,
			double unitPrice) {
		this.EAN = EAN;
		this.name = name;
		this.desc = desc;
		this.saleAmount = saleAmount;
		this.stockAmount = stockAmount;
		this.unitPrice = unitPrice;
		this.total = saleAmount * unitPrice;
	}

	
	
	//SaleItemDetail methods:
	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getEAN()
	  */
	@Override
	public String getEAN() {
		return EAN;
	}

	/** 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getPricePerUnit()
	  */
	@Override
	public double getPricePerUnit() {
		return unitPrice;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getProductDescription()
	  */
	@Override
	public String getProductDescription() {
		return desc;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getProductName()
	  */
	@Override
	public String getProductName() {
		return name;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getSaleAmount()
	  */
	@Override
	public double getSaleAmount() {
		return saleAmount;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getStockAmount()
	  */
	@Override
	public double getStockAmount() {
		return stockAmount;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#getTotalPrice()
	  */
	@Override
	public double getTotalPrice() {
		return total;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail#setSaleAmount(double)
	  */
	@Override
	public void setSaleAmount(double newSaleAmount) {
		saleAmount = newSaleAmount;
	}

}
