/* $Id: ControllerHelper.java 715 2009-09-20 21:11:30Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui;


import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.IEvents;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail;
//import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.debug.SaleItemDetailImpl;

public abstract class ControllerHelper {
	
	private static IEvents invocationTarget = null;
	private static Logger logger = Logger.getLogger(ControllerHelper.class);
	
	public static void initializeOnlyOnce(IEvents t)
	throws java.lang.IllegalStateException, java.lang.IllegalArgumentException {
		if (invocationTarget == null) {
			if (t == null) {
				throw new IllegalArgumentException("Null argument.");
			}
			invocationTarget = t;
		} else {
			throw new IllegalStateException("Already initialized.");
		}
	}
	
	/**
	  * 
	  * @return true if {@link #invocationTarget} is not <code>null</code>.
	  */
	private static boolean handleNullInvocationTarget() {
		if (invocationTarget == null) {
			Throwable t = new Throwable("");
			StackTraceElement[] els = t.getStackTrace();
			if (els.length > 1) {
				logger.error(
						els[1].getClass().getName() + "." + els[1].getMethodName() +
						": NULL invocation target.");
			} else {
				logger.error("ControllerHelper (unable to determine source): Null invocation targer.");
			}
		}
		return (invocationTarget != null); 
	}
	  
	public static void saleStarted() {
		if (handleNullInvocationTarget()) {
			invocationTarget.saleStarted();
		}
	}

	public static SaleItemDetail itemSold(String EAN, double amount) {
    	SaleItemDetail result = null;
    	if (handleNullInvocationTarget()) {
    		result = invocationTarget.itemSold(EAN, amount);
    	}
    	/*debug*/ //else { result = new SaleItemDetailImpl(EAN, amount); }
    	return result;
    }

    public static void saleItemUpdated(String EAN, double amount) {
    	if (handleNullInvocationTarget()) {
    		invocationTarget.saleItemUpdated(EAN, amount);
    	}
    }
    
    public static boolean startPayment(PaymentType pt, double total, double cash)
    throws PaymentRefusedException {
    	boolean result = false;
    	if (handleNullInvocationTarget()) {
   			result = invocationTarget.startPayment(pt, total, cash);
    	}
    	return result;
    }

    public static boolean cancelPayment() {
    	boolean result = true;
    	if (handleNullInvocationTarget()) {
    		result = invocationTarget.cancelPayment();
    	}
    	return result;
    }

    public static double getSum() {
    	double result = 0;
    	if (handleNullInvocationTarget()) {
    		result = invocationTarget.getSum();
    	}
    	return result;
    }

    public static void saleClosed() {
    	if (handleNullInvocationTarget()) {
    		invocationTarget.saleClosed();
    	}
    }

    public static boolean cancelSale() {
    	boolean result = true;
    	if (handleNullInvocationTarget()) {
    		result = invocationTarget.cancelSale();
    	}
    	return result;
	}
}
