/* $Id: IEvents.java 715 2009-09-20 21:11:30Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui;


/** Interface for event notification from GUI component.
  * 
  * @author Ondrej Cerny
  *
  */
public interface IEvents {
  /** Notifies that a sale has been started.
    */ 
  public void saleStarted();

  /** Notifies that an item has been sold and returns detailed information
    * about the item.
    * 
    * <p>
    * This call blocks the caller thread until the item is fetched from inventory.
    * </p>
    * 
    * @param EAN unique identification of the item
    * @param amount number of sold items
    * @returns detailed information about the item
    */
  public SaleItemDetail itemSold(String EAN, double amount);

  /** Notifies that a sale item with given EAN hase been updated.
    *
    * @param EAN identification of the updated sale item
    * @param amount new amount of goods
    */
  public void saleItemUpdated(String EAN, double amount);

  /** Asks for starting payment.
    *
    * <p>
    * This call blocks the caller thread until the payment is finished.
    * </p>
    *
    * @param pt payment type
    * @param total total to be payed by this payment
    * @param cash given sum (within non-cash payment, the <code>total</code>
    *             should be equal to and <code>cash</code>)
    * @return return true iff human confirmation is required for this payment
    */
  public boolean startPayment(PaymentType pt, double money, double cash)
  throws PaymentRefusedException;

  /** Asks for canceling payment.
    *
    * @returns true <i>iff</i> the payment has been successfully canceled
    *          (when false returned, the payment has been finished and it
    *           is not posible to cancel it any more)
    */
  public boolean cancelPayment();

  /** Returns sum of money to be payed yet.
    *
    * @return sum
    */
  public double getSum();

  /** Notifies that the sale has been closed (successfully finished).
    *
    */
  public void saleClosed();

  /** Asks for canceling sale.
    * 
    * @returns true <i>iff</i> the sale has been successfully canceled
    *          (when false returned, the sale has been finished and it
    *           is not posible to cancel it any more)
    */
  public boolean cancelSale();
}
