/* $Id: SaleItemDetail.java 710 2009-09-19 13:54:04Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui;

import java.io.Serializable;

/** Summarizes sale item details in a plain object.
  *
  * <p>
  * Contains following fields:
  * <ul>
  * <li>EAN</li>
  * <li>Product name</li>
  * <li>Product description</li>
  * <li>Sale amount</li>
  * <li>Stock amount</li>
  * <li>Price per unit</li>
  * <li>Total price (<code>Price per unit * Sale amount</code>)</li>
  * </ul>
  * </p>
  * 
  * @author Ondrej Cerny
  *
  */
public interface SaleItemDetail extends Serializable {
	
	/** Returns unique identification of stock item.
	  * 
	  * @return EAN
	  */
	public String getEAN();
	
	/** Returns product name.
	  * 
	  * @return product name
	  */
	public String getProductName();
	
	/** Returns product description. 
	  * 
	  * @return product description
	  */
	public String getProductDescription();
	
	/** Returns sale amount.
	  * 
	  * @return sale amount
	  */
	public double getSaleAmount();
	
	/** Returns stock amount.
	  * 
	  * @return stock amount
	  */
	public double getStockAmount();
	
	/** Returns unit price.
	  * 
	  * @return unit price
	  */
	public double getPricePerUnit();
	
	/** Returns total price for this sale item.
	  * 
	  * <p>
	  * Usually it is sale amount multiplied by unit price.
	  * </p>
	  * 
	  * @return total price for this sale item
	  */
	public double getTotalPrice();

	/** Sets sale amount.
	  * 
	  * @param newSaleAmount new sale amount
	  */
	public void setSaleAmount(double newSaleAmount);
}
