package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.swing;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.ControllerHelper;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.PaymentType;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.swing.ModeTabs.Mode;


public class PaymentPane extends JPanel implements ModePane, ActionListener, CaretListener {

	private static final long serialVersionUID = 1L;
	
	private static final int SIZE_X = 2;
	
	private PaymentType paymentType;
	private double total;

	private int currx = 0;
	private int curry = 0;
	
	private JLabel paymentTypeLabel = null;
	private JLabel validationLabel = null;
	private JLabel remainderLabel = null;
	private JTextField toPayInput = null;
	private JLabel changeLabel = null;
	private JButton startButton = null;
	private JButton cancelButton = null;

	

	/**
	 * This is the default constructor
	 */
	public PaymentPane() {
		super();
		initialize();
	}
	
	private GridBagConstraints getNextGridBagConstraint(int colls) {
		if (currx + colls > SIZE_X) {
			++curry;
			currx = 0;
		}
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.gridx = currx;
		c.gridy = curry;
		c.gridwidth = colls;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(10, 10, 10, 10);
		c.ipadx = 100;
		currx += colls;
		return c;
	}
	
	private void addToGrid(Component c, int colls) {
		add(c, getNextGridBagConstraint(colls));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		setLayout(new GridBagLayout());
				
		addToGrid(getPaymentTypeLabel(), SIZE_X);

		addToGrid(getValidationLabel(), SIZE_X);
		
		addToGrid(new JLabel("Total:"), 1);

		addToGrid(getRemainderLabel(), 1);

		addToGrid(new JLabel("Payment:"), 1);
		
		GridBagConstraints c = getNextGridBagConstraint(1);
		c.ipadx = 100;
		add(getToPayInput(), c);
		
		addToGrid(new JLabel("Change:"), 1);
		
		addToGrid(getChangeLabel(), 1);
		
		addToGrid(getStartButton(), 1);
		
		addToGrid(getCancelButton(), 1);
	}
	
	public JLabel getPaymentTypeLabel() {
		if (paymentTypeLabel == null) {
			paymentTypeLabel = new JLabel();
			paymentTypeLabel.setFont(paymentTypeLabel.getFont().deriveFont(Font.BOLD));
		}
		return paymentTypeLabel;
	}
	
	public JLabel getValidationLabel() {
		if (validationLabel == null) {
			validationLabel = new JLabel();
			validationLabel.setForeground(Color.RED);
		}
		return validationLabel;
	}
	
	public JLabel getRemainderLabel() {
		if (remainderLabel == null) {
			remainderLabel = new JLabel();
		}
		return remainderLabel;
	}
	
	public JTextField getToPayInput() {
		if (toPayInput == null) {
			toPayInput = new JTextField();
			toPayInput.addCaretListener(this);
		}
		return toPayInput;
	}
	public JLabel getChangeLabel() {
		if (changeLabel == null) {
			changeLabel = new JLabel("0.00");
		}
		return changeLabel;
	}
	
	public JButton getStartButton() {
		if (startButton == null) {
			startButton = new JButton("OK");
			startButton.addActionListener(
					new PaymentPaneStartPaymentListener(this));
		}
		return startButton;
	}
	
	public JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(this);
		}
		return cancelButton;
	}

	
	
	public double getInsertedSum() 
	throws IllegalStateException {
		double result = 0;		
		try {
			String v = getToPayInput().getText();
			result = Double.valueOf(v).doubleValue();
		} catch (Exception e) {
			throw new IllegalStateException("Malformed sum.");
		}
		return result;
	}

	public ModeTabs getModeTabs() {
		Container c = getParent();
		if (c instanceof ModeTabs) {
			return (ModeTabs)c;
		}
		throw new IllegalStateException("Unable to get mode tabs.");
	}

	/**
	  *
	  */
	public void showValidationMessage(String m) {
		if (m != null) {
			getValidationLabel().setText(m);
		}
	}
	
	public double getNewTotal() {
		total = ControllerHelper.getSum();
		return total;
	}
	
	public double getTotal() {
		return total;
	}
	
	public PaymentType getPaymentType() {
		return paymentType;
	}
	public void updateChange() { 
		double money = 0;
		boolean numberOk = false; 
		try {
			money = getInsertedSum();
			numberOk = true;
		} catch (IllegalStateException ise) {}
		if (numberOk) {
			getChangeLabel().setText(String.format("%6.2f", money - total));
		} else {
			getChangeLabel().setText("------.--");
		}
	}


	@Override
	public void selectThis(int thisIndex, int lastIndex, int subModeIndex) {
		paymentType = PaymentType.CASH;
		if (subModeIndex == 1) {
			paymentType = PaymentType.CARD;
		} else if (subModeIndex != 0) {
			System.err.printf("PaymentPane: Unable to determine payment type - CASH used.\n");
		}

		//String total = String.format("%6.2f", getNewTotal());
		double total = getNewTotal();
		getToPayInput().setText(String.format("%6.2f", total));
		getRemainderLabel().setText(String.format("%.2f", total));
		getChangeLabel().setText("");

		if (paymentType == PaymentType.CASH) {
			getPaymentTypeLabel().setText("Cash payment");
			showValidationMessage("");
			getStartButton().setEnabled(true);
			getToPayInput().setEnabled(true);
			updateChange();
		} else if (paymentType == PaymentType.CARD) {
			getPaymentTypeLabel().setText("Card payment");
			showValidationMessage("");			
			getStartButton().setEnabled(true);
			getToPayInput().setEnabled(false);			
		} else {
			System.err.printf("PaymentPane: Unkown payment type!\n");
			showValidationMessage("Error: unkown payment type.");
			getStartButton().setEnabled(false);
			getToPayInput().setEnabled(false);
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JButton src = null;
		Object o = e.getSource();
		if (o instanceof JButton) {
			src = (JButton)o;
		}
		if (src == getCancelButton()) {
			if (ControllerHelper.cancelPayment()) {
				// may cancel
				getModeTabs().setMode(Mode.SALE);				
			} else {
				getCancelButton().setEnabled(false);
			}
		} else {
			System.err.printf(
					"Error: PaymentPane: unable to detect action source.\n");
		}	
	}

	@Override
	public void caretUpdate(CaretEvent e) {
		updateChange();
	}	
}
