package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.SwingWorker;

import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.ControllerHelper;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.PaymentRefusedException;
import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.PaymentType;


public class PaymentPaneStartPaymentListener implements ActionListener {
	
	private class StartPaymentWorker extends SwingWorker<Object, Object> {
		private PaymentPane paymentPane;
		private PaymentType paymentType;
		private double payment;
		private double total;
		private double remainder;
		private PaymentRefusedException paymentRefused;
		
		protected StartPaymentWorker(PaymentPane pane, double payment) {
			this.paymentPane = pane;
			this.paymentType = pane.getPaymentType();
			this.payment = payment;
			this.total = pane.getTotal();
			this.remainder = 0;
			this.paymentRefused = null;
		}

		@Override
		protected Double doInBackground() throws Exception {
			try {
				// TODO check return value which should indicate whether 
				// the payment reuquires cashier confirmation
				ControllerHelper.startPayment(paymentType, total, payment);
				remainder = ControllerHelper.getSum();
				if (remainder == 0) {
					ControllerHelper.saleClosed();
				} else {
					System.err.printf("Remainder is %f.\n", remainder);
				}
			} catch (PaymentRefusedException pre) {
				paymentRefused = pre;
			}
			return Double.valueOf(remainder);
		}
				
		@Override
		protected void done() {
			if (remainder == 0 && paymentRefused == null) {
				// sale already closed, just swith modes ...
				paymentPane.getModeTabs().setMode(
						ModeTabs.Mode.START);
			} else {
				if (paymentRefused != null) {
					paymentPane.showValidationMessage("Payment refused: " +
							paymentRefused.getMessage());
					
				} else {
					// TODO: show this as error dialog box...
					System.err.printf("Internal error: " +
							"PaymentPaneStartPaymentListener: " +
							"ControllerHelper.startSale(pt, %f, %f) == %f.\n",
							total, payment, remainder);
				}
			}
		}
	}
	
	private PaymentPane paymentPane;
	
	public PaymentPaneStartPaymentListener(PaymentPane context) {
		paymentPane = context;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO: disable everything except Cancel button...
		
		try {
			paymentPane.showValidationMessage("");
			double payment = paymentPane.getInsertedSum();
			double total = paymentPane.getTotal();
			PaymentType pt = paymentPane.getPaymentType();
			if (payment < total) {
				paymentPane.showValidationMessage("Partial payment not supported.");
			} else if (payment != total && pt == PaymentType.CARD) {
				paymentPane.showValidationMessage("Cash back card payment not supported.");
			} else {
				StartPaymentWorker spw = new StartPaymentWorker(paymentPane, payment);
				spw.execute();
			}
		} catch (IllegalStateException isex) {
			paymentPane.showValidationMessage("Invalid payment sum.");
			//TODO enable all controls (appropriate to the current payment type)
		}
	}
}
