package org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.swing;

import java.util.Collection;
import java.util.List;
import java.util.LinkedList;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import org.objectweb.dsrg.sofa.demos.sofashop.cashdesk.gui.SaleItemDetail;

public class SaleTableModel extends AbstractTableModel implements TableModel {
	
	private String[] columnNames = {"EAN", "Name", "Amount", "Unit price", "Total price"};
	private List<SaleItemDetail> rows;
	private boolean editMode;
	private int editRow;
	
	public SaleTableModel() {
		rows = new LinkedList<SaleItemDetail>();
		editMode = false;
		editRow = -1;
	}

    public int getColumnCount() {
      return columnNames.length;
	}
    
    public int getRowCount() {
        return rows.size();
    }

    public boolean isDataEmpty() {
    	return (rows.size() == 0);
    }


    public String getColumnName(int col) {
        return columnNames[col];
    }
    
    private Object getSaleItemDetailField(SaleItemDetail sid, int col) {
    	switch (col) {
    	case 0: return sid.getEAN();
    	case 1: return sid.getProductName();
    	case 2: return String.format("%6.2f", sid.getSaleAmount());
    	case 3: return String.format("%6.2f", sid.getPricePerUnit());
    	case 4: return String.format("%6.2f", sid.getTotalPrice());
    	default: return "";
    	}
    }

    public Object getValueAt(int row, int col) {
    	SaleItemDetail sid = rows.get(row);
    	if (sid == null) {
    		return "";
    	} else {
    		return getSaleItemDetailField(sid, col);
    	}
    }

    public Class<?> getColumnClass(int c) {
    	return String.class;
    }

	public boolean isCellEditable(int row, int col) {
		if (editMode) {
			return (row == editRow) && (col == 1);
		}
		return false;
    }

    public void setValueAt(Object value, int row, int col) {
    	if (isCellEditable(row, col)) {
    		double newVal = 0;
    		boolean convertable = false;
            SaleItemDetail sid = rows.get(row);
            if (value instanceof Double) {
            	newVal = ((Double)value).doubleValue();
            	convertable = true;
            }
            if (convertable) {
            	sid.setSaleAmount(newVal);
            	fireTableCellUpdated(row, col);
            }
    	}
    }
    
    public void addRows(Collection<SaleItemDetail> sids) {
		int oldRowCount = getRowCount();
    	if (sids != null) {
    		for(SaleItemDetail sid: sids) {
    			if (sid != null) {
    				rows.add(sid);
    			}
    		}
    		int newRowCount = getRowCount();
    		if (newRowCount >= oldRowCount) {
        		fireTableRowsInserted(oldRowCount, newRowCount - 1);    			
    		}
    	}
    }
    
    public void deleteAllData() {
    	rows.clear();
    	fireTableDataChanged();
    }
    
}
