	/* $Id: AbstractObjectUpdater.java 805 2009-11-09 02:20:23Z ondrac $
	 *
	 * SofaShop demo for SOFA 2.0 component system
	 * Copyright (C) 2009  DSRG, Charles University in Prague
	 *                     http://dsrg.mff.cuni.cz/
	 *
	 * This library is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU Lesser General Public
	 * License as published by the Free Software Foundation; either
	 * version 2.1 of the License, or (at your option) any later version.
	 *
	 * This library is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	 * Lesser General Public License for more details.
	 *
	 * You should have received a copy of the GNU Lesser General Public
	 * License along with this library; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	 * MA 02110-1301 USA
	 *
	 */
		
	package org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.util;
	
	import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.*;
	
	/** Provides updating object by a storage non-aware object. 
	  *
	  * <p>
	  * This class was generated from ecore model by M2T template using
	  * Acceleo MDA Generator (see http://www.acceleo.org).
	  * </p>
	  *	  
	  */ 	
 	public abstract class AbstractObjectUpdater {
 	
		/** Notifies that new object has been created.
 	      *
 	      * @param o new object
 	      * @return given object (or its storage-aware copy)
 	      */
		protected abstract <T extends AbstractBase> T objectCreated(T o);
 	
	  			
		//
		// StockItem abstract stuff
		//
		
 	    /** Returns StockItem equivalent to given.
 	      *
 	      * @param o original StockItem
 	      * @returns current replica of the StockItem 
 	      */
		protected abstract StockItem getReplica_StockItem(StockItem o);

 	    /** Notifies that a StockItem object has been released.
 	      *
 	      * @param o contained StockItem
 	      */
		protected abstract void contentReleased_StockItem(StockItem o);
		
		//
		// StockItem update stuff
		//
	
			/** Returns updated replica of given StockItem. 
		  *
		  * @param newObject StockItem
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(StockItem newObject) {
			StockItem original = getReplica_StockItem(newObject);
			if (original == null) {
				original = new StockItem();
			}
			
			update_StockItem(original, newObject);
			
			return original;
		}

		/** Updates an original StockItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(StockItem originalObject, StockItem newObject) {
			update_StockItem(originalObject, newObject);
		}

		/** Updates an original StockItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_StockItem(StockItem originalObject, StockItem newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(StockItem originalObject, StockItem newObject) {
			originalObject.setEAN(newObject.getEAN());
			originalObject.setAmount(newObject.getAmount());
			originalObject.setPrice(newObject.getPrice());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(StockItem originalObject, StockItem newObject) {
			
			
			//
			// update reference product
			//
	 		{
	 			Product current = originalObject.getProduct();
				Product replacement = newObject.getProduct();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Product(replacement); 
							if (current == null) {
								current = new Product();
								update_Product(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setProduct(current);
			}
			
			
			//
			// update reference store
			//
	 		{
	 			Store current = originalObject.getStore();
				Store replacement = newObject.getStore();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Store(replacement); 
							if (current == null) {
								current = new Store();
								update_Store(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setStore(current);
			}
		}
		
	
	  			
		//
		// SaleItem abstract stuff
		//
		
 	    /** Returns SaleItem equivalent to given.
 	      *
 	      * @param o original SaleItem
 	      * @returns current replica of the SaleItem 
 	      */
		protected abstract SaleItem getReplica_SaleItem(SaleItem o);

 	    /** Notifies that a SaleItem object has been released.
 	      *
 	      * @param o contained SaleItem
 	      */
		protected abstract void contentReleased_SaleItem(SaleItem o);
		
		//
		// SaleItem update stuff
		//
	
			/** Returns updated replica of given SaleItem. 
		  *
		  * @param newObject SaleItem
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(SaleItem newObject) {
			SaleItem original = getReplica_SaleItem(newObject);
			if (original == null) {
				original = new SaleItem();
			}
			
			update_SaleItem(original, newObject);
			
			return original;
		}

		/** Updates an original SaleItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(SaleItem originalObject, SaleItem newObject) {
			update_SaleItem(originalObject, newObject);
		}

		/** Updates an original SaleItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_SaleItem(SaleItem originalObject, SaleItem newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(SaleItem originalObject, SaleItem newObject) {
			originalObject.setAmount(newObject.getAmount());
			originalObject.setPricePerUnit(newObject.getPricePerUnit());
			originalObject.setTotalPrice(newObject.getTotalPrice());
			originalObject.setStockItemEAN(newObject.getStockItemEAN());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(SaleItem originalObject, SaleItem newObject) {
			
			
			//
			// update reference product
			//
	 		{
	 			Product current = originalObject.getProduct();
				Product replacement = newObject.getProduct();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Product(replacement); 
							if (current == null) {
								current = new Product();
								update_Product(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setProduct(current);
			}
		}
		
	
	  			
		//
		// Sale abstract stuff
		//
		
 	    /** Returns Sale equivalent to given.
 	      *
 	      * @param o original Sale
 	      * @returns current replica of the Sale 
 	      */
		protected abstract Sale getReplica_Sale(Sale o);

 	    /** Notifies that a Sale object has been released.
 	      *
 	      * @param o contained Sale
 	      */
		protected abstract void contentReleased_Sale(Sale o);
		
		//
		// Sale update stuff
		//
	
			/** Returns updated replica of given Sale. 
		  *
		  * @param newObject Sale
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(Sale newObject) {
			Sale original = getReplica_Sale(newObject);
			if (original == null) {
				original = new Sale();
			}
			
			update_Sale(original, newObject);
			
			return original;
		}

		/** Updates an original Sale by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(Sale originalObject, Sale newObject) {
			update_Sale(originalObject, newObject);
		}

		/** Updates an original Sale by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_Sale(Sale originalObject, Sale newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(Sale originalObject, Sale newObject) {
			originalObject.setCashDeskNo(newObject.getCashDeskNo());
			originalObject.setClose(newObject.getClose());
			originalObject.setTotal(newObject.getTotal());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(Sale originalObject, Sale newObject) {
			
			
			//
			// update reference saleItems
			//
			{
				java.util.List<SaleItem> originalList = originalObject.getSaleItems();
				java.util.List<SaleItem> newList = newObject.getSaleItems();
				if (originalList == null) {
					originalList = new java.util.LinkedList<SaleItem>();
				}			
				int originalSize = originalList.size();
				int currentIndex = 0;		
				SaleItem current = null;
				if (newList != null) { 				 				
					for (SaleItem replacement : newList) {
						if (replacement != null) {
							current = null;
							if (currentIndex < originalSize) {
								current = originalList.get(currentIndex);
							}
														
							// update containment
							if (current == null) {
								current = new SaleItem();
								update_SaleItem(current, replacement);
								current = objectCreated(current);							
							} else {
								update_SaleItem(current, replacement);
							}
							

							if (currentIndex < originalSize) {
								originalList.set(currentIndex, current);
							} else {
								originalList.add(current);
							}
							++currentIndex;
						}
					}
				}
				while(originalSize > currentIndex) {
					--originalSize;
					current = originalList.remove(originalSize);
					if(current != null) { contentReleased_SaleItem(current); }
				}
				originalObject.setSaleItems(originalList);
			}
			
			
			//
			// update reference store
			//
	 		{
	 			Store current = originalObject.getStore();
				Store replacement = newObject.getStore();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Store(replacement); 
							if (current == null) {
								current = new Store();
								update_Store(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setStore(current);
			}
		}
		
	
	  			
		//
		// Product abstract stuff
		//
		
 	    /** Returns Product equivalent to given.
 	      *
 	      * @param o original Product
 	      * @returns current replica of the Product 
 	      */
		protected abstract Product getReplica_Product(Product o);

 	    /** Notifies that a Product object has been released.
 	      *
 	      * @param o contained Product
 	      */
		protected abstract void contentReleased_Product(Product o);
		
		//
		// Product update stuff
		//
	
			/** Returns updated replica of given Product. 
		  *
		  * @param newObject Product
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(Product newObject) {
			Product original = getReplica_Product(newObject);
			if (original == null) {
				original = new Product();
			}
			
			update_Product(original, newObject);
			
			return original;
		}

		/** Updates an original Product by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(Product originalObject, Product newObject) {
			update_Product(originalObject, newObject);
		}

		/** Updates an original Product by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_Product(Product originalObject, Product newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(Product originalObject, Product newObject) {
			originalObject.setDefaultEAN(newObject.getDefaultEAN());
			originalObject.setDefaultPrice(newObject.getDefaultPrice());
			originalObject.setName(newObject.getName());
			originalObject.setId(newObject.getId());
			originalObject.setDescription(newObject.getDescription());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(Product originalObject, Product newObject) {
			
			
			//
			// update reference category
			//
	 		{
	 			ProductCategory current = originalObject.getCategory();
				ProductCategory replacement = newObject.getCategory();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_ProductCategory(replacement); 
							if (current == null) {
								current = new ProductCategory();
								update_ProductCategory(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setCategory(current);
			}
			
			
			//
			// update reference supplyTerms
			//
			{
				java.util.List<SupplyTerms> originalList = originalObject.getSupplyTerms();
				java.util.List<SupplyTerms> newList = newObject.getSupplyTerms();
				if (originalList == null) {
					originalList = new java.util.LinkedList<SupplyTerms>();
				}			
				int originalSize = originalList.size();
				int currentIndex = 0;		
				SupplyTerms current = null;
				if (newList != null) { 				 				
					for (SupplyTerms replacement : newList) {
						if (replacement != null) {
							current = null;
							if (currentIndex < originalSize) {
								current = originalList.get(currentIndex);
							}
														
							// update containment
							if (current == null) {
								current = new SupplyTerms();
								update_SupplyTerms(current, replacement);
								current = objectCreated(current);							
							} else {
								update_SupplyTerms(current, replacement);
							}
							

							if (currentIndex < originalSize) {
								originalList.set(currentIndex, current);
							} else {
								originalList.add(current);
							}
							++currentIndex;
						}
					}
				}
				while(originalSize > currentIndex) {
					--originalSize;
					current = originalList.remove(originalSize);
					if(current != null) { contentReleased_SupplyTerms(current); }
				}
				originalObject.setSupplyTerms(originalList);
			}
		}
		
	
	  			
		//
		// Store abstract stuff
		//
		
 	    /** Returns Store equivalent to given.
 	      *
 	      * @param o original Store
 	      * @returns current replica of the Store 
 	      */
		protected abstract Store getReplica_Store(Store o);

 	    /** Notifies that a Store object has been released.
 	      *
 	      * @param o contained Store
 	      */
		protected abstract void contentReleased_Store(Store o);
		
		//
		// Store update stuff
		//
	
			/** Returns updated replica of given Store. 
		  *
		  * @param newObject Store
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(Store newObject) {
			Store original = getReplica_Store(newObject);
			if (original == null) {
				original = new Store();
			}
			
			update_Store(original, newObject);
			
			return original;
		}

		/** Updates an original Store by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(Store originalObject, Store newObject) {
			update_Store(originalObject, newObject);
		}

		/** Updates an original Store by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_Store(Store originalObject, Store newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(Store originalObject, Store newObject) {
			originalObject.setId(newObject.getId());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(Store originalObject, Store newObject) {
		}
		
	
	  			
		//
		// SupplyTerms abstract stuff
		//
		
 	    /** Returns SupplyTerms equivalent to given.
 	      *
 	      * @param o original SupplyTerms
 	      * @returns current replica of the SupplyTerms 
 	      */
		protected abstract SupplyTerms getReplica_SupplyTerms(SupplyTerms o);

 	    /** Notifies that a SupplyTerms object has been released.
 	      *
 	      * @param o contained SupplyTerms
 	      */
		protected abstract void contentReleased_SupplyTerms(SupplyTerms o);
		
		//
		// SupplyTerms update stuff
		//
	
			/** Returns updated replica of given SupplyTerms. 
		  *
		  * @param newObject SupplyTerms
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(SupplyTerms newObject) {
			SupplyTerms original = getReplica_SupplyTerms(newObject);
			if (original == null) {
				original = new SupplyTerms();
			}
			
			update_SupplyTerms(original, newObject);
			
			return original;
		}

		/** Updates an original SupplyTerms by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(SupplyTerms originalObject, SupplyTerms newObject) {
			update_SupplyTerms(originalObject, newObject);
		}

		/** Updates an original SupplyTerms by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_SupplyTerms(SupplyTerms originalObject, SupplyTerms newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(SupplyTerms originalObject, SupplyTerms newObject) {
			originalObject.setLowerLimitIncl(newObject.getLowerLimitIncl());
			originalObject.setUpperLimitExcl(newObject.getUpperLimitExcl());
			originalObject.setMaxDeliveryTime(newObject.getMaxDeliveryTime());
			originalObject.setPrice(newObject.getPrice());
			originalObject.setValidFrom(newObject.getValidFrom());
			originalObject.setValidTo(newObject.getValidTo());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(SupplyTerms originalObject, SupplyTerms newObject) {
			
			
			//
			// update reference supplier
			//
	 		{
	 			Supplier current = originalObject.getSupplier();
				Supplier replacement = newObject.getSupplier();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Supplier(replacement); 
							if (current == null) {
								current = new Supplier();
								update_Supplier(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setSupplier(current);
			}
			
			
			//
			// update reference store
			//
	 		{
	 			Store current = originalObject.getStore();
				Store replacement = newObject.getStore();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Store(replacement); 
							if (current == null) {
								current = new Store();
								update_Store(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setStore(current);
			}
		}
		
	
	  			
		//
		// Supplier abstract stuff
		//
		
 	    /** Returns Supplier equivalent to given.
 	      *
 	      * @param o original Supplier
 	      * @returns current replica of the Supplier 
 	      */
		protected abstract Supplier getReplica_Supplier(Supplier o);

 	    /** Notifies that a Supplier object has been released.
 	      *
 	      * @param o contained Supplier
 	      */
		protected abstract void contentReleased_Supplier(Supplier o);
		
		//
		// Supplier update stuff
		//
	
			/** Returns updated replica of given Supplier. 
		  *
		  * @param newObject Supplier
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(Supplier newObject) {
			Supplier original = getReplica_Supplier(newObject);
			if (original == null) {
				original = new Supplier();
			}
			
			update_Supplier(original, newObject);
			
			return original;
		}

		/** Updates an original Supplier by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(Supplier originalObject, Supplier newObject) {
			update_Supplier(originalObject, newObject);
		}

		/** Updates an original Supplier by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_Supplier(Supplier originalObject, Supplier newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(Supplier originalObject, Supplier newObject) {
			originalObject.setRegNo(newObject.getRegNo());
			originalObject.setName(newObject.getName());
			originalObject.setDescription(newObject.getDescription());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(Supplier originalObject, Supplier newObject) {
		}
		
	
	  			
		//
		// ProductCategory abstract stuff
		//
		
 	    /** Returns ProductCategory equivalent to given.
 	      *
 	      * @param o original ProductCategory
 	      * @returns current replica of the ProductCategory 
 	      */
		protected abstract ProductCategory getReplica_ProductCategory(ProductCategory o);

 	    /** Notifies that a ProductCategory object has been released.
 	      *
 	      * @param o contained ProductCategory
 	      */
		protected abstract void contentReleased_ProductCategory(ProductCategory o);
		
		//
		// ProductCategory update stuff
		//
	
			/** Returns updated replica of given ProductCategory. 
		  *
		  * @param newObject ProductCategory
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(ProductCategory newObject) {
			ProductCategory original = getReplica_ProductCategory(newObject);
			if (original == null) {
				original = new ProductCategory();
			}
			
			update_ProductCategory(original, newObject);
			
			return original;
		}

		/** Updates an original ProductCategory by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(ProductCategory originalObject, ProductCategory newObject) {
			update_ProductCategory(originalObject, newObject);
		}

		/** Updates an original ProductCategory by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_ProductCategory(ProductCategory originalObject, ProductCategory newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(ProductCategory originalObject, ProductCategory newObject) {
			originalObject.setName(newObject.getName());
			originalObject.setId(newObject.getId());
			originalObject.setDescription(newObject.getDescription());
			originalObject.setLevel(newObject.getLevel());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(ProductCategory originalObject, ProductCategory newObject) {
			
			
			//
			// update reference parent
			//
	 		{
	 			ProductCategory current = originalObject.getParent();
				ProductCategory replacement = newObject.getParent();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_ProductCategory(replacement); 
							if (current == null) {
								current = new ProductCategory();
								update_ProductCategory(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setParent(current);
			}
		}
		
	
	  			
		//
		// OrderItem abstract stuff
		//
		
 	    /** Returns OrderItem equivalent to given.
 	      *
 	      * @param o original OrderItem
 	      * @returns current replica of the OrderItem 
 	      */
		protected abstract OrderItem getReplica_OrderItem(OrderItem o);

 	    /** Notifies that a OrderItem object has been released.
 	      *
 	      * @param o contained OrderItem
 	      */
		protected abstract void contentReleased_OrderItem(OrderItem o);
		
		//
		// OrderItem update stuff
		//
	
			/** Returns updated replica of given OrderItem. 
		  *
		  * @param newObject OrderItem
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(OrderItem newObject) {
			OrderItem original = getReplica_OrderItem(newObject);
			if (original == null) {
				original = new OrderItem();
			}
			
			update_OrderItem(original, newObject);
			
			return original;
		}

		/** Updates an original OrderItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(OrderItem originalObject, OrderItem newObject) {
			update_OrderItem(originalObject, newObject);
		}

		/** Updates an original OrderItem by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_OrderItem(OrderItem originalObject, OrderItem newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(OrderItem originalObject, OrderItem newObject) {
			originalObject.setAmount(newObject.getAmount());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(OrderItem originalObject, OrderItem newObject) {
			
			
			//
			// update reference terms
			//
	 		{
	 			SupplyTerms current = originalObject.getTerms();
				SupplyTerms replacement = newObject.getTerms();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_SupplyTerms(replacement); 
							if (current == null) {
								current = new SupplyTerms();
								update_SupplyTerms(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setTerms(current);
			}
			
			
			//
			// update reference product
			//
	 		{
	 			Product current = originalObject.getProduct();
				Product replacement = newObject.getProduct();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Product(replacement); 
							if (current == null) {
								current = new Product();
								update_Product(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setProduct(current);
			}
		}
		
	
	  			
		//
		// Order abstract stuff
		//
		
 	    /** Returns Order equivalent to given.
 	      *
 	      * @param o original Order
 	      * @returns current replica of the Order 
 	      */
		protected abstract Order getReplica_Order(Order o);

 	    /** Notifies that a Order object has been released.
 	      *
 	      * @param o contained Order
 	      */
		protected abstract void contentReleased_Order(Order o);
		
		//
		// Order update stuff
		//
	
			/** Returns updated replica of given Order. 
		  *
		  * @param newObject Order
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(Order newObject) {
			Order original = getReplica_Order(newObject);
			if (original == null) {
				original = new Order();
			}
			
			update_Order(original, newObject);
			
			return original;
		}

		/** Updates an original Order by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(Order originalObject, Order newObject) {
			update_Order(originalObject, newObject);
		}

		/** Updates an original Order by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_Order(Order originalObject, Order newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(Order originalObject, Order newObject) {
			originalObject.setCode(newObject.getCode());
			originalObject.setCreateDate(newObject.getCreateDate());
			originalObject.setApproveDate(newObject.getApproveDate());
			originalObject.setCloseDate(newObject.getCloseDate());
			originalObject.setCancelDate(newObject.getCancelDate());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(Order originalObject, Order newObject) {
			
			
			//
			// update reference orderItems
			//
			{
				java.util.List<OrderItem> originalList = originalObject.getOrderItems();
				java.util.List<OrderItem> newList = newObject.getOrderItems();
				if (originalList == null) {
					originalList = new java.util.LinkedList<OrderItem>();
				}			
				int originalSize = originalList.size();
				int currentIndex = 0;		
				OrderItem current = null;
				if (newList != null) { 				 				
					for (OrderItem replacement : newList) {
						if (replacement != null) {
							current = null;
							if (currentIndex < originalSize) {
								current = originalList.get(currentIndex);
							}
														
							// update containment
							if (current == null) {
								current = new OrderItem();
								update_OrderItem(current, replacement);
								current = objectCreated(current);							
							} else {
								update_OrderItem(current, replacement);
							}
							

							if (currentIndex < originalSize) {
								originalList.set(currentIndex, current);
							} else {
								originalList.add(current);
							}
							++currentIndex;
						}
					}
				}
				while(originalSize > currentIndex) {
					--originalSize;
					current = originalList.remove(originalSize);
					if(current != null) { contentReleased_OrderItem(current); }
				}
				originalObject.setOrderItems(originalList);
			}
			
			
			//
			// update reference supplier
			//
	 		{
	 			Supplier current = originalObject.getSupplier();
				Supplier replacement = newObject.getSupplier();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Supplier(replacement); 
							if (current == null) {
								current = new Supplier();
								update_Supplier(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setSupplier(current);
			}
			
			
			//
			// update reference store
			//
	 		{
	 			Store current = originalObject.getStore();
				Store replacement = newObject.getStore();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Store(replacement); 
							if (current == null) {
								current = new Store();
								update_Store(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setStore(current);
			}
		}
		
	
	  			
		//
		// ProductPolicy abstract stuff
		//
		
 	    /** Returns ProductPolicy equivalent to given.
 	      *
 	      * @param o original ProductPolicy
 	      * @returns current replica of the ProductPolicy 
 	      */
		protected abstract ProductPolicy getReplica_ProductPolicy(ProductPolicy o);

 	    /** Notifies that a ProductPolicy object has been released.
 	      *
 	      * @param o contained ProductPolicy
 	      */
		protected abstract void contentReleased_ProductPolicy(ProductPolicy o);
		
		//
		// ProductPolicy update stuff
		//
	
			/** Returns updated replica of given ProductPolicy. 
		  *
		  * @param newObject ProductPolicy
		  * @return updated repplica of newObject
		  */
		public Object getUpdatedReplica(ProductPolicy newObject) {
			ProductPolicy original = getReplica_ProductPolicy(newObject);
			if (original == null) {
				original = new ProductPolicy();
			}
			
			update_ProductPolicy(original, newObject);
			
			return original;
		}

		/** Updates an original ProductPolicy by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		public void update(ProductPolicy originalObject, ProductPolicy newObject) {
			update_ProductPolicy(originalObject, newObject);
		}

		/** Updates an original ProductPolicy by a new one.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void update_ProductPolicy(ProductPolicy originalObject, ProductPolicy newObject) {
			updateFields(originalObject, newObject);
			updateAllReferences(originalObject, newObject);
		}
		
	
		/** Updates field values.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateFields(ProductPolicy originalObject, ProductPolicy newObject) {
			originalObject.setLowLimit(newObject.getLowLimit());
			originalObject.setEnoughLimit(newObject.getEnoughLimit());
			originalObject.setValidFrom(newObject.getValidFrom());
			originalObject.setValidTo(newObject.getValidTo());
		}

			
		/** Update refrences.
		  *
		  * @param originalObject original object
		  * @param newObject new object
		  */
		protected void updateAllReferences(ProductPolicy originalObject, ProductPolicy newObject) {
			
			
			//
			// update reference product
			//
	 		{
	 			Product current = originalObject.getProduct();
				Product replacement = newObject.getProduct();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Product(replacement); 
							if (current == null) {
								current = new Product();
								update_Product(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setProduct(current);
			}
			
			
			//
			// update reference store
			//
	 		{
	 			Store current = originalObject.getStore();
				Store replacement = newObject.getStore();
	 			if (replacement == null) {
	 				current = null;	 		
	 				
	 			} else {
												
							// update association
							current = getReplica_Store(replacement); 
							if (current == null) {
								current = new Store();
								update_Store(current, replacement);
								current = objectCreated(current);	
							}
							

				}
				originalObject.setStore(current);
			}
		}
		
	
		
		//
		// catch
		//
		/** Throws ClassCastException.
		  */
		public void update(Object o1,Object o2) 
		throws ClassCastException {
			if (o1 != null && o2 != null && (!o1.getClass().equals(o2.getClass()))) {
				throw new ClassCastException(
					"Incompatible objects.");
			}
			throw new ClassCastException(
				"Objects are not model-members.");
		}
				
	}		

