/* $Id: ClientInsertTrigger.java 644 2009-08-19 16:07:34Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.db.storage;

import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.neodatis.odb.OID;
import org.neodatis.odb.core.trigger.InsertTrigger;
import org.apache.log4j.Logger;

/** Inserts unicate id to each inserted object.
  *
  * @author Ondrej Cerny
  */
public class ClientInsertTrigger extends InsertTrigger {

  /** Logger. */
  private static Logger logger = Logger.getLogger(ClientInsertTrigger.class);

  /** Updates object's internal id.
    *
    * @param object object to be inserted
    */
  public boolean beforeInsert(Object object) {
    if (object != null && object instanceof AbstractBase) {
      if (AbstractBase.isInternalObjectIdInvalid((AbstractBase)object)) {
        AbstractBase.setInternalObjectId((AbstractBase)object, ODBWrapper.internalObjectIdManager.getNextId());
      }
    } // else it is an object that will stay disconnected once passed away
    return true;
  }

  /** Does nothing, required by super class.
    *
    * In fact, this method should be never called as observed.
    */
  public void afterInsert(Object object, OID oid) {
  }

}
