/* $Id: ClientSelectTrigger.java 644 2009-08-19 16:07:34Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.db.storage;

import java.util.Set;
import java.util.HashSet;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.neodatis.odb.OID;
import org.neodatis.odb.core.trigger.SelectTrigger;
import org.apache.log4j.Logger;

/** Stores OIDs of selected objects to a set.
  *
  * This trigger should be applied only to sessions that require
  * knowledge of OIDs of fethced objects (managed transactions).
  *
  * @author Ondrej Cerny 
  */
public class ClientSelectTrigger extends SelectTrigger {
  /** Logger. */
  private static Logger logger = Logger.getLogger(ClientSelectTrigger.class);
  /** Set of OIDs */
  private Set<OID> setOfIds = null;

  /** Creates new OID set.
    *
    * @return new OID set
    */
  private Set<OID> newOidSet() {
    return new HashSet<OID>();
  }

  /** Creates new client-side select trigger.
    */
  public ClientSelectTrigger() {
    this.setOfIds = newOidSet();
  }

  /** Adds OID of currently selected object to the set of OIDs.
    *
    * @param object object representation
    */
  public void afterSelect(Object object, OID oid) {
    if (oid != null) {
      if (setOfIds != null && !setOfIds.contains(oid)) {
        setOfIds.add(oid);
      }
    } else {
      /* this should be treated as serious error
       * but all exceptions from here are ignored by
       * ODB engine
       */
      logger.fatal("OID of selected object is null.");
    }
  }

  /** Get OID set and clear it internally.
    * 
    * @return set of selected OIDs
    */
  public Set<OID> getSetAndClear() {
    Set<OID> tmp = this.setOfIds;
    this.setOfIds = newOidSet();
    return tmp;
  }
}

