/* $Id: ObjectStorage.java 652 2009-08-24 00:17:16Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.db.storage;

import java.util.Set;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.neodatis.odb.core.query.IQuery;
import org.neodatis.odb.OID;

/** Provides access to object storage.
  *
  * @author Ondrej Cerny 
  */
public interface ObjectStorage {

  /** Opens session at the object storage.
    *
    * @param managed if the session is managed, each query will return
    *   set of object ids as well as the objects itselves<br/>
    *   it may be useful for managing transactions but brings some extra
    *   overhead<br/>
    *   if the session is not managed, set of ids in QueryResult will be
    *   always null
    * @return session id required by all other methods
    */
  public int open(boolean managed);

  /** Fetches objects matching query <code>q</code>.
    * 
    * @param sessionId session identification
    * @param q query
    * @param startIndex index of the first returned object
    * @param endIndex index of the first object behind the last returned object
    * @return object containing collection of object and set of their OIDs
    */
  public <T extends AbstractBase> QueryResult<T> fetchObjects(int sessionId, IQuery q, int startIndex, int endIndex);

  /** Disconnects objects from the session.
    * 
    * <p>
    * This might be useful for managing transactions
    * and/or reusing sessions.
    * </p>
    *
    * @param sessionId session identification
    * @param setOfIds set of object ids
    */
  public void disconnectObjects(int sessionId, Set<OID> setOfIds);

  /** Stores object.
    * 
    * @param sessionId session identification
    * @param object object
    * @return stored object (just the returned instance MUST be used after the
    *         store within the same transaction)
    *
    */
  public <T extends AbstractBase> T storeObject(int sessionId, T object);

  /** Commits the changes and closes the session.
    *
    * @param sessionId session identification
    */
  public void commitAndClose(int sessionId);

  /** Ignores the changes and closes the session.
    *
    * @param sessionId session identification
    */
  public void rollbackAndClose(int sessionId);
}
