/* $Id: ReadOnlyTransactionHelper.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.db.transaction;

import org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.storage.ObjectStorage;
import org.objectweb.dsrg.sofa.demos.sofashop.db.storage.QueryResult;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.neodatis.odb.Objects;
import org.neodatis.odb.core.query.IQuery;
import org.apache.log4j.Logger;

public class ReadOnlyTransactionHelper extends TransactionHelperBase
implements ISelect {
  /** Logger. */
  private static Logger logger = Logger.getLogger(ReadOnlyTransactionHelper.class);
  /** Maximal number of fetches per session. */
  private static final int MAX_FETCHES_PER_SESSION = 100;

  /** Counts fetches per session. */
  private int nFetches;

  public ReadOnlyTransactionHelper(ObjectStorage o) {
    super(o, false);
    nFetches = 0;
  }

  //TODO: to stay up to date, we should close and reopen connection
  //      after writes - the writes must be somehow notified by
  //      the transaction manager

  /** Notifies up-coming data fetch.
    *
    * <p>
    * Closes connection after MAX_FETCHES_PER_SESSION fetches.
    * This prevents huge memory consumption.
    * </p>
    */
  private void notifyUpcomingFetch() {
    ++nFetches;
    if (MAX_FETCHES_PER_SESSION <= nFetches) {
      objectStorageProvider.commitAndClose(getTransactionNumber());
      endTransaction();
      nFetches = 0;
    }
    getTransactionNumber();
  }

  // ISelect methods:

  /** Implements
    * {@link org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect#select(IQuery) }
    * method.
    */ 
  public <T extends AbstractBase> Objects<T> select(IQuery i)
  throws DatabaseUnreachableException {
    return select(i, -1, -1);
  }

  /** Implements
    * {@link org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect#select(IQuery, int, int) }
    * method.
    */ 
  public <T extends AbstractBase> Objects<T> select(IQuery i, int startIndex, int endIndex)
  throws DatabaseUnreachableException {
    int n = getTransactionNumber();
    if (n < 0) {
      throw new DatabaseUnreachableException("Unable to open database session.");
    }
    // notify up-coming fetch (re-initializes session sometimes)
    notifyUpcomingFetch();  
    // fetch the data 
    QueryResult<T> qr = objectStorageProvider.fetchObjects(n, i, startIndex, endIndex);

    if (qr != null) {
      return qr.getObjects();
    } else {
      return null;
    }
  }
  
  @Override
  public boolean updateTimeStamp(int currentTimeStamp) {
    boolean result = super.updateTimeStamp(currentTimeStamp);
    logger.trace("call: ReadOblyTransactionHelper::updateTimeStamp()");
    if (!result) {
    	endTransaction();
    	logger.trace("Re-initializing read-only transaction to get up to date (history: " + nFetches + " data fetches).");
    	nFetches = 0;
    }
    return result;
  }

}
