/* $Id: TransactionHelperBase.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.db.transaction;

import org.objectweb.dsrg.sofa.demos.sofashop.db.storage.ObjectStorage;

public abstract class TransactionHelperBase {
  /** Database access provider. */
  protected ObjectStorage objectStorageProvider;
  /** Current transaction number.
    *
    * <p>Negative number indicates invalid or no transaction.</p>
    */
  private int transNo;
  /** Time stamp of the last transaction (re)opening. 
    * 
    */
  private int timeStamp;
  /** Are transactions handled with this helper 'managed'.
    *
    * Managed transaction allow to monitor fetched objects.
    */
  private boolean managed;
  /** Constructs BaseTransactionHelper.
    *
    * @param objectStorageProvider object database provider 
    * @param managed are transactions handled with this helper 'managed'?
    */
  protected TransactionHelperBase(ObjectStorage objectStorageProvider, boolean managed) {
    this.objectStorageProvider = objectStorageProvider;
    this.managed = managed;
    transNo = -1;
  }
  /** Gets current transaction number (or starts new transaction).
    *
    * @return transaction number
    */
  protected int getTransactionNumber() {
    if (transNo < 0) {
      // no transaction has been started, start one
      transNo = objectStorageProvider.open(managed);
    }
    return transNo;
  }
  /** Ends transaction (after commit or rollback).
    *
    * <p>Next call of getTransactionNumber will create new transaction.</p>
    */
  protected void endTransaction() {
    transNo = -1;
  }
  /** Indicates whether a transaction is active. 
    *
    * @return true if and only if a transaction is active
    */
  protected boolean isActive() {
    return (transNo >= 0);
  }

  /** Sets {@link #timeStamp} to the given one and return false if they were not equal.
    * 
    * @param currentTimeStamp
    * @return false if and only if the given time stamp and {@link #timeStamp}
    *         were not equal just before this call
    */
  public boolean updateTimeStamp(int currentTimeStamp) {
    boolean result = false;
    synchronized (this) {
      if (this.timeStamp != currentTimeStamp) {
          // update this time stamp
    	  // !!! it is up to a subclass, to re-initialize the database connection
    	  this.timeStamp = currentTimeStamp;
    	  result = false;
      }
    }
    return result;
  }
}
