/* $Id: AuthenticationHelper.java 763 2009-10-16 23:16:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.de;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.IQuery;
import org.neodatis.odb.impl.core.query.criteria.CriteriaQuery;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Store;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect;
import org.objectweb.dsrg.sofa.demos.sofashop.db.InvalidContextException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.TransactionAbortedException;
import org.objectweb.dsrg.sofa.demos.sofashop.de.Credentials;
import org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange;

/**
 * @author Ondrej Cerny
 *
 */
public class AuthenticationHelper {
	
	private static Logger logger = Logger.getLogger(AuthenticationHelper.class);
	private static int MAX_RETRY_ATTEMTPS = 100; 
	
	private static int handleUnsuccessfullAttempt(int attempts, Exception e)
	throws DatabaseUnreachableException {
		if (e instanceof DatabaseUnreachableException) {
			throw (DatabaseUnreachableException)e;
		}
		if (e instanceof InvalidContextException || e instanceof TransactionAbortedException) {
			if (attempts > MAX_RETRY_ATTEMTPS) {
				throw new DatabaseUnreachableException("Database unreachable.", e);
			}
			return ++attempts;
		}
		throw new DatabaseUnreachableException("Non-database error.", e);
	}
	
	/** Authenticates caller bound to given databse (and store)
	  * with given passphrase to given data exchange server.
	  * 
	  * @param deServer data exchange server
	  * @param database local database (for getting store info)  
	  * @param passphrase passphrase
	  * @return session id (zero indicates failure)
	  * @throws DatabaseUnreachableException
	  */
	public static long authenticate(
			IDataExchange deServer, ISelect database, long storeId, String passphrase)
			throws DatabaseUnreachableException {
		if (database == null) {
			throw new DatabaseUnreachableException("Database not initialized properly.");
		}
		if (deServer == null) {
			throw new DatabaseUnreachableException("Data exchange serber not initialized properly.");			
		}
		if (passphrase == null) {
			return 0;
		}
		Store currentStore = null;
		/*
		IQuery qry = new CriteriaQuery(Store.class);
		Collection<Store> stores = null;
		int attempts = 0;
		while (stores == null) {
			try {
				stores = database.select(new CriteriaQuery(Store.class));
				if (stores == null) {
					break;
				}
			} catch (InvalidContextException ice) {
				attempts = handleUnsuccessfullAttempt(attempts, ice);
			}
		}
		if (stores != null && stores.size() > 0) {
			boolean hit = false;
			for(Store s: stores) {
				if (currentStore == null) {
					currentStore = s;
				}
				if (storeId == s.getId()) {
					currentStore = s;
					hit = true;
					break;
				} else {
					logger.error("Having another store in store database. " +
							"Please check the configuration or clean database.");
					// TODO stop
				}
			}
			if (!hit) {
				logger.info("Using firts store.");
			}
		} else {
			logger.ingo("No store in database. Probably this is the first run.");
		}*/
		if (currentStore == null) {
			currentStore = new Store();
			currentStore.setId(storeId);			
		}
		long result = deServer.authenticate(new Credentials(currentStore, passphrase));
		if (result != 0) {
			logger.info("Authenticated.");
		}
		
		if (result == 0) {		
			logger.info("Authentication failed.");
		}
		return result;
	}
}
