/* $Id: DataDownloadAction.java 813 2009-11-17 12:05:05Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.de;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.criteria.ICriterion;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Order;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.ITransact;
import org.objectweb.dsrg.sofa.demos.sofashop.db.TransactionAbortedException;
import org.objectweb.dsrg.sofa.demos.sofashop.de.DataExchangeRefusedException;
import org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange;

/** Dowloads data from data exchange server.
  * 
  * @author Ondrej Cerny
  *
  */
public class DataDownloadAction implements DataExchangeAction {
	
	private static Logger logger = Logger.getLogger(DataDownloadAction.class);
	private ITransact tx;
	private IDataExchange de;
	private long sessionId;
	/** Creates down-loader.
	  * 
	  * @param localDatabase local database tx provider
	  * @param remoteExchanger remote exchange component
	  */
	public DataDownloadAction(ITransact localDatabase, IDataExchange remoteExchanger) {
		tx = localDatabase;
		de = remoteExchanger;
	}
	
	/** Performs down-load of objects of given class and matching given
	  * criterion.
	  * 
	  * <p>
	  * Only downloads of {@link Product products} and {@link Order orders}
	  * are supported by now.
	  * </p>
	  * 
	  * @param clazz class
	  * @param crit criterion
	  * @throws InventoryUnreachableException on error
	  */
	public void perform(Class<? extends AbstractBase> clazz, ICriterion crit)
	throws DatabaseUnreachableException {
		Collection<? extends AbstractBase> result;		
		try {
			logger.trace("Call downloadObjects");
			result = de.downloadObjects(this.sessionId, clazz, crit);
		} catch (DataExchangeRefusedException dere) {
			throw new DatabaseUnreachableException(
				"Data exchange refused.", dere);
		}
		
		if (result == null) {
			throw new DatabaseUnreachableException(
					"Data exchange not successfull.");
		}
		
		// use updater to update objects
		DataExchangeClientObjectUpdater decou = new DataExchangeClientObjectUpdater(tx);
		TransactionAbortedException tae = null;
		for(AbstractBase object: result) {
			for(int i = 0; i < 10; ++i) {
				tae = null;
				try {
					decou.updateObjectInDatabaseByObject(
							object);
				} catch (TransactionAbortedException txae) {
					//logger.trace("Transaction aborted: " + tae.getMessage());
					tae = txae;
				}
			}
		}
		if (tae != null) {
			throw new DatabaseUnreachableException(
					"Unable to process transaction.",
					tae);
		}
		
		
	}

	@Override
	public void setSessionId(long sessionId) {
		this.sessionId = sessionId;
	}
}
