/* $Id: IDataExchange.java 766 2009-10-17 23:52:31Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.de;

import java.util.Collection;
import java.util.List;
import java.io.Serializable;

import org.neodatis.odb.Objects;
import org.neodatis.odb.core.query.criteria.ICriterion;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;

/** Data exchange interface.
  * 
  * @author Ondrej Cerny
  *
  */
public interface IDataExchange {
	
	/** Authenticates client.
	  * 
	  * @param credentials
	  * @return session id (zero indicates failure)
	  */
	public long authenticate(Credentials credentials);

	/** Downloads objects from DataExchangeServer.
	  *  
	  * @param sessionId session id (returned by {@link #authenticate(Credentials)})  
	  * @param clazz class
	  * @param criteria criteria
	  * @return objects
	  * @throws DatabaseUnreachableException on database error
	  * @throws DataExchangeRefusedException on authentication or authorisation error
	  */
	public <T extends AbstractBase> Objects<T> downloadObjects(
			long sessionId, Class<T> clazz, ICriterion criteria)
			throws DatabaseUnreachableException, DataExchangeRefusedException;

	/** Uploads objects to DataExchangeServer.
	  * 
	  * @param objects objects
	  * @throws DatabaseUnreachableException on database error
	  * @throws DataExchangeRefusedException on authentication or authorisation error
	  */
	public <O extends AbstractBase> void uploadObjects(long sessionId, List<O> objects)
			throws DatabaseUnreachableException, DataExchangeRefusedException;
}
