/* $Id: RequestManager.java 837 2009-11-26 04:53:45Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.de;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.criteria.ICriterion;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;

/**
  * @author Ondrej Cerny
  *
  */
public class RequestManager {
	/** Map of previous download times.
	  * 
	  * <p>
	  * Maps <i>downloaded class</i> &rarr; <i>time stamp</i>.
	  * </p> 
	  */
	private Map<Class<? extends AbstractBase>, Long> lastDownloads = new HashMap<Class<? extends AbstractBase>, Long>();
	/** Minimal time between downloads in seconds. */
	public int minDownloadPeriod = 3600; 
	
	/** Logger. */
	private static Logger logger = Logger.getLogger(RequestManager.class);
	/** Does this objects confirm the exchange?.
	  * 
	  * @param clazz
	  * @param criterion
	  * @param direction
	  * @return
	  */
	public boolean exchangeConfirmed(Class<? extends AbstractBase> clazz,
					ICriterion criterion, int direction) {
		if (direction == IInitExchange.DIRECTION_GET) {
			return downloadConfirmed(clazz, criterion);
		} else if (direction == IInitExchange.DIRECTION_PUT) {
			return uploadConfirmed(clazz, criterion);
		} else {
			logger.error("Unknown data exchange request direction " +
					"(download/upload). Exchange refused.");
			return false;
		}
	}
	protected boolean downloadConfirmed(Class<? extends AbstractBase> clazz,
					ICriterion criterion) {
		boolean confirmed = true;
		Long timeStamp = lastDownloads.get(clazz);
		Calendar current = Calendar.getInstance();
		// get last download time
		if (timeStamp != null) {
			long nextTimeStamp = timeStamp.longValue() + minDownloadPeriod * 1000;
			logger.trace("Comparing TS: NOW=" + current.getTimeInMillis() +
					" ?<? " + nextTimeStamp + "=NEXT");
			if (current.getTimeInMillis() < nextTimeStamp) {
				logger.trace("NOW < NEXT --> refuse");
				confirmed = false;
			}
		}
		// update map -- add record only for entire class downloads
		if (confirmed) {
			if (criterion == null) {
				logger.trace("Adding record for current download.");
				lastDownloads.put(clazz, new Long(current.getTimeInMillis()));
			}
		}
		logger.trace("Download of " + clazz.getName() + " " +
				((criterion != null) ? ("where " + criterion) + " " : ("")) +
				(confirmed ? " confirmed." : " ignored (requested too often)."));
		return confirmed;
	}
	protected boolean uploadConfirmed(Class<? extends AbstractBase> clazz,
			        ICriterion criterion) {
		return true;
	}
}
