/* $Id: IOrder.java 762 2009-10-12 11:19:00Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.inv.order;

import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Order;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.InventoryUnreachableException;



public interface IOrder {

	/** Validates given {@link Order order} and saves it onto database.
	  * 
	  * @param o {@link Order}
	  * @returns {@link Order order} code (see {@link Order#getCode()})
	  * @throws OrderInconsistentException when the {@link Order order}
	  *         is not consistent
	  * @throws InventoryUnreachableException when any low level error
	  *         occurred
	  */
	public String registerOrder(Order o)
		throws InventoryUnreachableException, OrderInconsistentException; 
	
	/** Sets the order as received (good delivered).
	  *
	  * <p>
	  * Also updates or creates appropriate stock items.
	  * </p>
	  * 
	  * @param o {@link Order}
	  * @throws OrderInconsistentException when the {@link Order order}
	  *         does not exist or is inconsistent (already received).
	  * @throws InventoryUnreachableException when any low level error
	  *         occured
	  */
	public void receiveOrder(String orderCode)
		throws InventoryUnreachableException, OrderInconsistentException;

}
