/* $Id: IListProducts.java 762 2009-10-12 11:19:00Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.inv.prod;

import org.neodatis.odb.Objects;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.InventoryUnreachableException;

/**
  * 
  * @author Ondrej Cerny
  *
  */
public interface IListProducts {
	/** Returns products matching criterion.
	  * 
	  * @param categoryId id of category where all products must belong to
	  * @param namePatterm product name must be like this parameter
	  *                    (if null, all names will match)
	  * @param startIndex index of the first returned object
	  * @param endIndex index of the first object behind
	  *        the last returned object
	  * @return {@link Product products}
	  */
	public Objects<Product> listProducts(
			long categoryId, String namePattern, int startIndex, int endIndex)
			throws InventoryUnreachableException;
	/** Returns products matching criterion.
	  * 
	  * @param namePatterm product name must be like this parameter
	  *                    (if null, all names will match)
	  * @param startIndex index of the first returned object
	  * @param endIndex index of the first object behind
	  *        the last returned object
	  * @return {@link Product products}
	  */
	public Objects<Product> listProducts(
			String namePattern, int startIndex, int endIndex)
			throws InventoryUnreachableException;
	/** Returns products with given id.
	  *
	  * @param productId id of the product
	  * @return {@link Product products}
	  */
	public Product getProductById(
			long productId)
			throws InventoryUnreachableException;
	
}
