/* $Id: IListStock.java 652 2009-08-24 00:17:16Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.inv.stock;

import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.*;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.InventoryUnreachableException;
import org.neodatis.odb.Objects;

/** Provides listing of stock items.
  *
  * @author Ondrej Cerny
  */
public interface IListStock {
  /** Returns single StockItem uniquely identified by its EAN.
    *
    * @param EAN unique identification of StockItem
    * @return StockItem on success, <code>null</code> iff there is no such
    *         StockItem in the database
    */
  public StockItem getStockItemByEAN(String EAN)
    throws InventoryUnreachableException;

  /** Returns StockItems associated with a product which match exactly given
    * id.
    * 
    * @param id unique product identification
    * @param startIndex index of the first returned object
    * @param endIndex index of the first object behind the last returned object
    * @return A collection of StockItems
    */
  public Objects<StockItem> listStockItems(long id, int startIndex, int endIndex)
    throws InventoryUnreachableException;

  /** Returns StockItems associated with products with description that march
    * given pattern.
    *
    * <p>
    * Regular expresions (particulary the format used by java.util.regex) are
    * accepted within <code>pattern</code> argument.
    * </p>
    *
    * @param pattern regular expresion to be matched against product description
    * @param startIndex index of the first returned object
    * @param endIndex index of the first object behind the last returned object
    * @return A collection of of StockItems
    */
  public Objects<StockItem> listStockItems(String pattern,
                                           int startIndex, int endIndex)
    throws InventoryUnreachableException;
}
