/* $Id: StockItemsByProductDescQuery.java 652 2009-08-24 00:17:16Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.inv.stock;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.neodatis.odb.core.query.nq.NativeQuery;

import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.StockItem;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;

/** Fetches just StockItems associated to a Product with description
  * that matches pattern.
  * 
  * @author Ondrej Cerny
  */
public class StockItemsByProductDescQuery extends NativeQuery {

  /** The product id. */
  private Pattern pattern;

  /** Creates the query and sets the pattern.
    * 
    * @param pattern pattern to be matched within product description
    */
  public StockItemsByProductDescQuery(String pattern)
  throws IllegalArgumentException, PatternSyntaxException {
    this.pattern = Pattern.compile(pattern,
      Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
  }

  /** Examines the <code>obj</code> object for the query condition.
    *
    * @param obj object to be examined
    * @return true if and only if the <code>obj</code> object is an instance
    *         of StockItem and is associated to a Product with given product id.
    */
  public boolean match(java.lang.Object obj) {
    boolean result = false;
    if (obj instanceof StockItem) {
      Product p = ((StockItem) obj).getProduct();
      if (p != null && p.getDescription() != null) {
        result = pattern.matcher(p.getDescription()).find();
      }
    }
    return result;
  }

  /** Returns query object type.
    * 
    * @return query result class
    */
  public Class getObjectType() {
    return StockItem.class;
  }
}


