package org.objectweb.dsrg.sofa.demos.sofashop.pad;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.neodatis.odb.Objects;
import org.neodatis.odb.core.query.criteria.ICriterion;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Store;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.de.Credentials;
import org.objectweb.dsrg.sofa.demos.sofashop.de.DataExchangeRefusedException;
import org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange;

public class DummyDataExchangeServer implements IDataExchange {
	
	private static Logger logger = Logger.getLogger(DummyDataExchangeServer.class);
	
	Map<Long, Store> sessionMap = new HashMap<Long, Store>();
	long lastUsedSessionId = 0;
	boolean nextRun = false;
	
	/** Returns an available session id (including 0).
	  * 
	  * @return session id
	  */
	private synchronized long getUncheckedSessionId() {
			long previousLasUsedSessionId = lastUsedSessionId;
			++lastUsedSessionId;
			if (previousLasUsedSessionId > lastUsedSessionId) {
				nextRun = true;
			}
			while (nextRun) {
				if (!sessionMap.containsKey(new Long(lastUsedSessionId))) {
					break;
				}
				++lastUsedSessionId;
			}
			return lastUsedSessionId;
	}
	
	/** Returns an available non-zero session ID.
	  * 
	  * @return non-zero session id
	  */
	private long getAvailableSessionId() {
		long id = getUncheckedSessionId();
		if (id == 0) {
			id = getUncheckedSessionId();
		}
		if (id == 0) {
			throw new IllegalStateException(
					"Impossible happend: all session id's are occupied!");
		}
		return id;
	}
	
	/** Authenticates the client and assigns a session ID to it.
	  * 
	  * @return non-zero session id on success
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange#authenticate(org.objectweb.dsrg.sofa.demos.sofashop.de.Credentials)
	  */
	@Override
	public long authenticate(Credentials credentials) {
		long id = 0;
		if (credentials != null) {
			Store s = credentials.getStore();
			if (s != null) {
				id = getAvailableSessionId();
				
				// TODO : check password (on success, id is non-zero)
				
				if (id != 0) {
					synchronized (sessionMap) {
						sessionMap.put(new Long(id), s);
					}
					logger.info("Authentication of store " + s.getId() + " succeed. Session ID " + id + ".");
				} else {
					logger.info("Authentication of store " + s.getId() + " failed. Not authorized.");
				}
			} else {
				logger.info("Unable to associate session with null store.");
			}
		}
		return id;
	}
	
	/** Returns objects given by the criteria.
	  * 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange#downloadObjects(long, java.lang.Class, org.neodatis.odb.core.query.criteria.ICriterion)
	  */
	@Override
	public <T extends AbstractBase> Objects<T> downloadObjects(long sessionId,
			Class<T> clazz, ICriterion criteria)
			throws DatabaseUnreachableException, DataExchangeRefusedException {
		
		if (sessionMap.containsKey(sessionId)) {
			Store s = sessionMap.get(sessionId);
			
			if (s != null) {
			
				// TODO - get objects from database
			
				logger.info("Returning simple array of objects.");
				return TestObjectsFactory.create(clazz, s);
			} else {
				logger.info("Session ID " + sessionId + " associated with null store.");
			}
			
		} else {
			logger.info("Unregistered session ID " + sessionId);
		}
		return null;
	}

	/**
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.de.IDataExchange#uploadObjects(java.util.List)
	  */
	@Override
	public <O extends AbstractBase> void uploadObjects(long sessionId, List<O> ojects)
		throws DatabaseUnreachableException,
		       DataExchangeRefusedException {
		throw new DataExchangeRefusedException("Not implemented within pad.");
	}	
}
