package org.objectweb.dsrg.sofa.demos.sofashop.pad;

import java.util.Calendar;
import java.util.Date;
import java.util.LinkedList;

import org.apache.log4j.Logger;
import org.neodatis.odb.Objects;
import org.neodatis.odb.impl.core.query.list.objects.SimpleList;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Store;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Supplier;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;

public class TestObjectsFactory {
	
	private static Logger logger = Logger.getLogger(TestObjectsFactory.class);
	
	private static int counter = 0;
	@SuppressWarnings("unchecked")	
	public static <T extends AbstractBase> Objects<T> create(Class<T> clazz, Store s) {
		if (Product.class.equals(clazz)) {
			logger.debug("Creating objects of " + clazz.getName() + " with counter value " + counter);
			Objects<Product> prods = new SimpleList<Product>();
			
			fillProductsList(prods, s, ++counter);
			

			return (Objects<T>)prods;
			
		} else {
			logger.debug("Creating objects of " + clazz.getName() + " not implemented yet.");
		}
		return null;
	}
	
	private static void fillProductsList(Objects<Product> list, Store store, int shift) {
		Product p;
		Supplier s;
		
		s = new Supplier();
		s.setRegNo("CZ-S121");
		s.setName("Fanda");
		s.setDescription("Fanda - shift " + Integer.toString(shift));
		
		p = new Product();
		p.setId(1);
		p.setDefaultEAN("0001");
		p.setDefaultPrice(shift * 954);
		p.setName("Product 1");
		p.setDescription("Product 1 - shift " + Integer.toString(shift));
		
		LinkedList<SupplyTerms> stList = new LinkedList<SupplyTerms>();
		
		// as many supply terms as the shift value is big
		for (; shift > 0; --shift) {
			SupplyTerms st = new SupplyTerms();
			Date d = null;
			st.setLowerLimitIncl(10 * shift);
			st.setUpperLimitExcl(10 * (shift + 1));
			st.setPrice(900 * shift);
			st.setSupplier(s);
			st.setStore(store);
			Calendar c = Calendar.getInstance();
			
			c.set(2009, 8, 1);
			d = new Date(c.getTimeInMillis());
			logger.trace("setting date from = " + d.toString());			
			st.setValidFrom(d);
			
			c.set(2010, 8, 1);			
			d = new Date(c.getTimeInMillis());
			logger.trace("setting date to = " + d.toString());			
			st.setValidTo(d);			
			stList.add(st);
		}
		p.setSupplyTerms(stList);
		
		list.add(p);
	}
}
