/* $Id: ReportDataProviderImpl.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.IQuery;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.AbstractBase;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect;
import org.objectweb.dsrg.sofa.demos.sofashop.db.InvalidContextException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProcessingException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider;

/**
 * @author Ondrej Cerny
 *
 */
public class ReportDataProviderImpl implements ReportDataProvider {
	
	private static Logger logger = Logger.getLogger(ReportDataProviderImpl.class);
	
	private ISelect databse;
	private static final int MAX_ATTEMPTS = 100;
	
	public ReportDataProviderImpl(ISelect databse) {
		this.databse = databse; 
	}
	/** 
	  * 
	  * @return collection of selected data (never returns null, rather throws data processing exception)
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider#select(org.neodatis.odb.core.query.IQuery)
	  */
	@Override
	public synchronized <T extends AbstractBase> Collection<T> select(IQuery query)
	throws ReportDataProcessingException {
		int attempts = 0;
		try {
			while (attempts < MAX_ATTEMPTS) {
				try {
					logger.trace("Report data provider - calling qry " + query.toString());
					Collection<T> result = databse.select(query);
					if (result == null) {
						throw new ReportDataProcessingException("Null data received from database - database error.");
					}
					return result;
				} catch (InvalidContextException ice) {
					logger.trace("Invalid context exception.");
					++attempts; 
				}
			}
		} catch (DatabaseUnreachableException due) {
			logger.trace("Error during report data processing: " + due.getClass().getName() + ": " + due.getMessage());
			throw new ReportDataProcessingException("Database is unreachable.");
		}
		return null;
	}

}
