/* $Id: ReportSelector.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.samplereports.OrderReport;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.samplereports.OverviewOfOrdersReport;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.samplereports.SalesReport;

/**
  * @author Ondrej Cerny
  *
  */
public class ReportSelector {
	private static final String ORDER_HIDDEN_REPORT_NAME = "Order";	
	private static final String SALEOVERVIEW_REPORT_NAME = "Overview of sales";
	private static final String ORDEROVERVIEW_REPORT_NAME = "Overview of orders";		
	private Set<String> reportNameSet = new HashSet<String>();
	
	public ReportSelector() {
		reportNameSet.add(SALEOVERVIEW_REPORT_NAME);
		reportNameSet.add(ORDEROVERVIEW_REPORT_NAME);
	}
	
	public Report getReportByName(String name) {
		if (SALEOVERVIEW_REPORT_NAME.equals(name)) {
			return new SalesReport();
		} else if (ORDEROVERVIEW_REPORT_NAME.equals(name)) {
			return new OverviewOfOrdersReport();
		} else if (ORDER_HIDDEN_REPORT_NAME.equals(name)) {
			return new OrderReport();
		}
		return null;
	}
	public Set<String> getReportNames() {
		return Collections.unmodifiableSet(reportNameSet);
	}
}
