/* $Id: AbstractReportParameter.java 770 2009-10-19 10:30:55Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common;

import java.io.Serializable;

/** Final part of {@link ReportParameter}.
  * 
  * @author Ondrej Cerny
  *
  */
public abstract class AbstractReportParameter implements ReportParameter {
	
	private static final long serialVersionUID = -5747796431539260624L;
	/** Parameter name. */
	protected final String name;
	/** Parameter prompt (user readable parameter name) */
	protected final String prompt;
	/** Default value. */
	protected final ReportParameterLabeledValue defaultValue;
	/** Is null value allowed?. */
	protected final boolean isNullAllowed;
	/** Parameter value class. */
	protected final Class<? extends Serializable> valueClass; 
	
	/** Creates new instance.
	  * 
	  * @param name report name
	  * @param prompt report prompt
	  * @param type value object class
	  * @param isNullAllowed is null allowed
	  * @param defaultValue default value
	  */
	public AbstractReportParameter(String name, String prompt, Class<? extends Serializable> type, 
			boolean isNullAllowed, ReportParameterLabeledValue defaultValue) {
		this.name = name;
		this.prompt = prompt;
		this.isNullAllowed = isNullAllowed;
		this.defaultValue = defaultValue;
		this.valueClass = type;
	}
	
	/** Returns default labeled value.
	  * 
	  * @return {@link #defaultValue} field 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#getDefaultValue()
	  */
	@Override
	public ReportParameterLabeledValue getDefaultValue() {
		return defaultValue; 
	}

	/** Return parameter name.
	  *
	  * @return {@link name} field
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#getName()
	  */
	@Override
	public String getName() {
		return name;
	}

	/** Returns parameter prompt.
	  *
	  * @return {@link prompt} field
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#getPrompt()
	  */
	@Override
	public String getPrompt() {
		return prompt;
	}
	/** Has this parameter a default value?
	  *
	  * @return value of {@link #hasDefaultValue} field 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#hasDefaultValue()
	  */
	@Override
	public boolean hasDefaultValue() {
		return defaultValue != null;
	}

	/** Is null value allowed?
	  * 
	  * @return value of {@link #isNullAllowed} field
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#isNullaAllowed()
	  */
	@Override
	public boolean isNullAllowed() {
		return isNullAllowed;
	}
	
	/** Returns type of value.
	  * 
	  * @return type
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter#getType()
	  */
	@Override
	public Class<? extends Serializable> getType() {
		return valueClass;
	}


}
