/* $Id: Report.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common;

import java.io.Serializable;
import java.util.List;

/**
 * @author Ondrej Cerny
 *
 */
public interface Report extends Serializable {
	
	/** Sets report data provider.
	  *
	  * <p>
	  * When the report is processed, this method is called always before
	  * any other call of this interface. Note that this data provider may
	  * become invalid, after each call of {@link #getParameters()},
	  * {@link #getParameters(List)} or {@link #getPrint(List)}.
	  * So it should be no longer held.
	  * </p>
	  * 
	  * @param p {@link ReportDataProvider}
	  */
	public void setDataProvider(ReportDataProvider p);
	
	/** Returns list of parameters.
	  *
	  * <p>
	  * The point is to provide initial state of parameters.
	  * </p>
	  * 
	  * @return list of parameters
	  * @throws ReportDataProcessingException when any error occured during data processing	
	  */
	public List<ReportParameter> getParameters()
	throws ReportDataProcessingException;
	/** Returns list of parameters based on their current values.
	  * 
	  * <p>
	  * The point is to update available values (if dependent on
	  * other parameters' selected values).
	  * </p> 
	  * 
	  * @param currentValues current values of parameters
	  * @throws ReportDataProcessingException when any error occured during data processing	  * @return list of parameters
	  */
	public List<ReportParameter> getParameters(List<ReportParameter> currentValues)
	throws ReportDataProcessingException;
	
	/**	Returns report print for given parameters.
	  * 
	  * @param values
	  * @return {@link ReportPrint} extension
	  * @throws InvalidReportParameterException when any parameter is invalid
	  * @throws ReportDataProcessingException when any error occured during data processing
	  */
	public ReportPrint getPrint(List<ReportParameter> values)
	throws InvalidReportParameterException, ReportDataProcessingException;
	
}
