/* $Id: ReportParameterLabeledValue.java 770 2009-10-19 10:30:55Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common;

import java.io.Serializable;

/** 
  * @author Ondrej Cerny
  *
  */
public class ReportParameterLabeledValue implements Serializable {
	private static final long serialVersionUID = 6770022679863216073L;
	/** Parameter value label (text representation to be shown to user). */
	private final String label;
	/** Parameter value. */
	private final Serializable value;
	
	public ReportParameterLabeledValue(final Serializable value, final String label) {
		this.label = label;
		this.value = value;
	}
	
	/** Returns parameter label (text representation to be shown to user). 
	  * 
	  * @return {@link #label} field 
	  */
	public String getLabel() {
		return label;
	}
	/** Returns parameter value.
	  * 
	  * @return {@link #value} field
	  */
	public Serializable getValue() {
		return value;
	}
	
	/** Returns label of this value.
	  * 
	  */
	@Override
	public String toString() {
		String result = null;
		if (this.label == null) {
			if (this.value == null) {
				result = "";
			} else {
				if (value == null) {
					result = "";
				} else {
					this.value.toString();
				}
			}
		} else {
			result = this.label;
		}
		return result;
	}
	
	/** Defines equity based on value.
	  * 
	  */
	@Override
	public boolean equals(Object o) {
		boolean result = false;
		if (o != null && o instanceof ReportParameterLabeledValue) {
			ReportParameterLabeledValue v = (ReportParameterLabeledValue)o;
			result = v.getValue() == this.getValue() // same reference (or both null)
				|| (this.getValue() != null && this.getValue().equals(v.getValue())); 
		}
		return result;
	}
}
