/* $Id: AbstractReportingServiceClient.java 770 2009-10-19 10:30:55Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.service;

import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;

/**
 * @author Ondrej Cerny
 *
 */
public abstract class AbstractReportingServiceClient {
	
	private static Logger logger = Logger.getLogger(AbstractReportingServiceClient.class);
	
	/** Returns reporting service provider. 
	  *
	  * <p>
	  * Must be thread safe as well as the {@link ReportingService} implementation.
	  * </p>
	  * 
	  * @return {@link ReportingService}
	  */
	protected abstract ReportingService getSerivceProvider();
	
	/** Returns report.
	  * 
	  * @param name report name (code name)
	  * @return {@link ReportProxy} or null
	  */
	public Report getReportForName(String name) {
		logger.trace("GetReportForName");
		ReportingService rs = getSerivceProvider();
		int id = rs.openReport(name);
		if (id < 0) {
			logger.trace("GetReportForName: Reporting service returned invalid id. Returning null.");
			return null;
		}
		logger.trace("GetReportForName: Creating report proxy for id " + Integer.toString(id));
		return new ReportProxy(id, rs);
	}
	
	/** Releases report.
	  *
	  * <p>
	  * Notifies {@link ReportingService} that it can releas the report (see {@link ReportingService#closeReport(int)}).
	  * </p>
	  * 
	  * @param p report (just {@link ReportProxy} will take an effect)
	  */
	public void releaseReport(Report p) {
		logger.trace("ReleaseReport");
		if (p!= null && p instanceof ReportProxy) {
			logger.trace("ReleaseReport: id " + Integer.toString(((ReportProxy)p).getId()));
			getSerivceProvider().closeReport(((ReportProxy)p).getId());
		} else {
			logger.error("ReleaseReport: Report type mismatch (ReportProxy expected).");
		}
	}

	/** Returns list of report names.
	  * 
	  * @return list of available report names
	  */
	public Set<String> getReportNames() {
		return getSerivceProvider().getReportNames();
	}
}
