/* $Id: ReportProxy.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.service;

import java.util.List;

import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.InvalidReportParameterException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProcessingException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportPrint;

/**
 * @author Ondrej Cerny
 *
 */
public class ReportProxy implements Report {
	
	int targetId;
	ReportingService serviceProvider;
	
	protected ReportProxy(int id, ReportingService serviceProvider) {
		this.targetId = id;
		this.serviceProvider = serviceProvider;
	}
	
	protected int getId() {
		return targetId;
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters()
	 */
	@Override
	public List<ReportParameter> getParameters() 
	throws ReportDataProcessingException {
		return serviceProvider.getReportParameters(targetId);
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters(java.util.List)
	 */
	@Override
	public List<ReportParameter> getParameters(
			List<ReportParameter> currentValues) 
	throws ReportDataProcessingException {
		return serviceProvider.getReportParameters(targetId, currentValues);
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getPrint(java.util.List)
	 */
	@Override
	public ReportPrint getPrint(List<ReportParameter> values)
			throws InvalidReportParameterException, ReportDataProcessingException {
		return serviceProvider.getReportPrint(targetId, values);
	}
	
	/**
	  * 
	  * <p>
	  * This implementation does not need any data provider.
	  * </p>
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#setDataProvider(org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider)
	  */
	@Override
	public void setDataProvider(ReportDataProvider p) {
	}
}
