/* $Id: ParameterValueConversionHelper.java 770 2009-10-19 10:30:55Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.util;

import java.io.Serializable;
import java.lang.reflect.Constructor;

import org.apache.log4j.Logger;

/**
 * @author Ondrej Cerny
 *
 */
public abstract class ParameterValueConversionHelper {
	private static Logger logger = Logger.getLogger(ParameterValueConversionHelper.class); 
	
	public static <T extends Serializable> T valueOf(Class<T> clazz, String value) {
		if (clazz == null || value == null) {
			logger.trace("Null clazz or value; returning null.");
			return null;
		}
		try {
			Constructor<T> stringConstructor = null;
 			try {
				 stringConstructor = clazz.getConstructor(String.class);
			} catch (NoSuchMethodException nsme) {
				logger.warn("Parameter value of type " + clazz.getName() + " is not constructable from string. Returning null.");
				return null;			
			}
			
			logger.trace("Creating new instance of parameter value of type " + clazz.getName() + " from string '" + value + "'.");
			try {
				return stringConstructor.newInstance(value);
			} catch (Throwable t) {
				logger.warn("Instantiation failed with " + t.getClass().getName() + ": " +t.getMessage());
				logger.trace("Returning null.");
				return null;
			}
		} catch (Throwable t) {
			logger.warn("Error during converting string to parameter value. Returning null. (" + t.getClass().getName() + ": " + t.getMessage() + ")");
			return null;
		} 
	}

}
