/* $Id: RemovableReportPane.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.util;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * @author Ondrej Cerny
 *
 */
public class RemovableReportPane extends JPanel implements ActionListener, Runnable {
	private RemovableReportContainer container;
	
	/** Creates report linked to its container.
	  * 
	  * @param reportPane
	  */
	public RemovableReportPane(RemovableReportContainer container, JPanel reportPane) {
		super();
		this.container = container;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());
		JPanel closeButtonPane = new JPanel();
		closeButtonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(this);
		closeButtonPane.add(closeButton);
		closeButton.setEnabled(container != null);
		add(closeButtonPane);
		add(reportPane);		
	}
	/** Schedules calling {@link RemovableReportContainer#removeReport(JPanel) } with this as argument later.
	  * 
	  * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	  */
	@Override
	public void actionPerformed(ActionEvent e) {
		SwingUtilities.invokeLater(this);
	}
	
	/** Calls {@link RemovableReportContainer#removeReport(JPanel) } with this as argument.
	  * 
	  * @see java.lang.Runnable#run()
	  */
	@Override
	public void run() {
		container.removeReport(this);
	}
}
