/* $Id: ReportPaneFactory.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.util;

import javax.swing.BoxLayout;
import javax.swing.JEditorPane;
import javax.swing.JPanel;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportPrint;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.SwingReportPrint;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.TextReportPrint;


/**
 * @author Ondrej Cerny
 *
 */
public abstract class ReportPaneFactory {
	
	private static Logger logger = Logger.getLogger(ReportPaneFactory.class);

	/** Creates a JPanel containing report print.
	  * 
	  * @param rp {@link ReportPrint}
	  * @return {@link JPanel}
	  */
	public static JPanel createReporPane(ReportPrint rp) {
		logger.trace("call");
		JPanel result = null;
		if (rp != null) {
			if (rp instanceof TextReportPrint) {
				logger.trace("text report");
				String content = ((TextReportPrint)rp).render();
				String contnetType = ((TextReportPrint)rp).getTextType();
				if (contnetType == null || contnetType.length() == 0) {
					contnetType = "text/html";
				}
				if (content == null) {
					content = "";
				}
				//logger.trace("contentType=\"" + contnetType + "\", text = \"" + content + "\"");
				JEditorPane ep = new JEditorPane(contnetType , content);
				ep.setEditable(false);
				result = new JPanel();
				result.setLayout(new BoxLayout(result, BoxLayout.Y_AXIS));
				result.add(ep);
				
			} else if (rp instanceof SwingReportPrint) {
				logger.trace("swing report");				
				result = ((SwingReportPrint)rp).render();
			} else {
				logger.trace("unknown report");				
				JEditorPane ep = new JEditorPane("text/html" ,"<body><h1>Report format not supported.</h1><p>The version of report viewer is probably outdated.</p></body>");
				ep.setEditable(false);
				result = new JPanel();
				result.setLayout(new BoxLayout(result, BoxLayout.Y_AXIS));
				result.add(ep);				
			}
		} else {
			logger.trace("swing report");							
			JEditorPane ep = new JEditorPane("text/html" ,"<body></body>");
			ep.setEditable(false);
			result = new JPanel();
			result.setLayout(new BoxLayout(result, BoxLayout.Y_AXIS));
			result.add(ep);
		}
		logger.trace("return");
		return result;
	}
	/** Creates a JPanel containing report and links it to a container where it should placed.
	  *
	  * <p>
	  * {@link RemovableReportContainer#removeReport(JPanel) } will be called with the result
	  * as parameter once user clicks a button for closing the report which is embedded in
	  * the result panel.
	  * </p>
	  *
	  * @param container {@link RemovableReportContainer} 
	  * @param rp {@link ReportPrint}
	  * @return {@link JPanel}
	  */	
	public static JPanel createClosableReportPane(RemovableReportContainer container, ReportPrint rp) {
		if (rp == null) {
			logger.trace("Null report print - returning null.");
			return null;
		}
		JPanel report = createReporPane(rp);
		return createClosableReportPaneFromExistingReportPane(container, report);
	}
	/** Creates a JPanel containing report and links it to a container where it should placed.
	  *
	  * <p>
	  * {@link RemovableReportContainer#removeReport(JPanel) } will be called with the result
	  * as parameter once user clicks a button for closing the report which is embedded in
	  * the result panel.
	  * </p>
	  *
	  * @param container {@link RemovableReportContainer} 
	  * @param report {@link JPanel} with report
	  * @return {@link JPanel}
	  */		
	public static JPanel createClosableReportPaneFromExistingReportPane(RemovableReportContainer container, JPanel report) {
		if (report != null) {
			return new RemovableReportPane(container, report);
		} else {
			logger.trace("Null report pane - returning null.");
			return null;
		}
	}
}
