/* $Id: ReportParameterIntputContainerPane.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.common.util;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingWorker;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProcessingException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter;

/**
 * @author Ondrej Cerny
 *
 */
public class ReportParameterIntputContainerPane extends JPanel {
	
	private static Logger logger = Logger.getLogger(ReportParameterIntputContainerPane.class);
	
	LinkedList<ParameterInput> activeInputs = new LinkedList<ParameterInput>();			
	List<ReportParameter> parameters = null;
	JPanel paramInputsPane = null;
	Report report = null;
	boolean parametersInProgress = false;
	
	public ReportParameterIntputContainerPane() {
		init();
	}
	
	private JPanel getParamInputsPane() {
		if (paramInputsPane == null) {
			paramInputsPane = new JPanel();
			paramInputsPane.setLayout(new GridLayout(0, 2));
		}
		return paramInputsPane;
	}
	
	private void init() {
		setLayout(new GridBagLayout());
		GridBagConstraints c = null;
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.NORTHWEST;
		add(getParamInputsPane(), c);
		// padding
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.gridheight = 1;
		add(new JPanel(), c);
	}
	
	/** Sets current report.
	  * 
	  * @param r report
	  */
	public void setReport(Report r) {
		logger.trace("set report");
		parameters = null;
		report = r;		
		parameterSelectionChanged(null);
		
	}
	
	protected ReportParameterIntputContainerPane getMe() {
		return this;
	}
	
	public void setAllInputsEnabled(boolean val) {
		if (activeInputs != null) {
			for(ParameterInput pi : activeInputs) {
				pi.setControlEnabled(val);
			}
		}
	}
	
	/** Notifies that current value of <i>rp</i> has been changed.
	  * 
	  * <p>
	  * Calls {@link Report#getParameters(List) } with current values as parameter
	  * and updates UI with the returned values. 
	  * </p>
	  * 
	  * @param rp report parameter or null if this is an initial load
	  */
	public void parameterSelectionChanged(ReportParameter rp) {
		if (report != null) {
			logger.trace("Parameter selection changed.");
			setAllInputsEnabled(false);
			setParametersInProgress(true);			
			
			SwingWorker<List<ReportParameter>, Object> sw = new SwingWorker<List<ReportParameter>, Object>() {
				@Override
				protected List<ReportParameter> doInBackground()
						throws Exception {
					logger.trace("Calling report for new param values.");
					try {
						if (parameters != null && parameters.size() > 0) {
							return report.getParameters(parameters);
						} else {
							return report.getParameters();
						}
					} catch (ReportDataProcessingException rdpe) {
						logger.info("Report data processing error: " + rdpe);
						return null;
					}
				}
				@Override
				protected void done() {
					List<ReportParameter> newParams = null;
					try {
						newParams = get();
					} catch (Throwable t) {
						logger.warn("Exception occured when obtaining " +
								"result of parameters exchange. " +
								"Exceptin: " + t.getClass().getName() +
								": " + t.getMessage());
					}
					if (newParams != null) {
						logger.trace("updating all parameter values (by that obtained from report)"); 						
						/*debug*/for (ReportParameter p: newParams) { logger.trace("obtained value of parameter '" + (p != null && p.getName() != null ? p.getName() : "<null>") + "' is '" + (p != null && p.getValue() != null && p.getValue().getLabel() != null ? p.getValue().getLabel() : "<null>") + "' ('" + (p != null && p.getValue() != null && p.getValue().getValue() != null && p.getValue().getValue().toString() != null ? p.getValue().getValue().toString() : "<null>") + "')"); }																								
						Iterator<ReportParameter> it = newParams.iterator();
						// update parameter inputs ...
						for (ParameterInput pi: activeInputs) {
							if (it.hasNext()) {
								pi.update(it.next());
								if (!pi.isDisplayed()) {
									getParamInputsPane().add(pi);
								}
							} else {
								getParamInputsPane().remove(pi);
								pi.setIsDisplayed(false);
								logger.error("Number of parameters changed decreased - not supported event!");
								break;
							}
						}
						while (it.hasNext()) {
							ReportParameter p = it.next();
							if (p != null) {
								ParameterInput pi = new ParameterInput(getMe());
								pi.update(p);
								activeInputs.add(pi);
								pi.setIsDisplayed(true);								
								getParamInputsPane().add(pi);
								logger.trace("Added parameter " + 
										(p.getName() != null ? p.getName() : "<null-name>") +
										" of type '" +
										(p.getType() != null ? p.getType().getName() : "<null-type>") +
										"'.");
							}
						}
						// update parameters...
						parameters = new LinkedList<ReportParameter>(newParams);
						/*debug*/for (ReportParameter p: parameters) { logger.trace("Current value of parameter '" + (p != null && p.getName() != null ? p.getName() : "<null>") + "' is '" + (p != null && p.getValue() != null && p.getValue().getLabel() != null ? p.getValue().getLabel() : "<null>") + "' ('" + (p != null && p.getValue() != null && p.getValue().getValue() != null && p.getValue().getValue().toString() != null ? p.getValue().getValue().toString() : "<null>") + "')"); }												
						updateUI();						
					} else {
						logger.trace("Null parameters obtained from report " +
								"as a result of parameter exchange.");
					}
					logger.trace("Parameters updated.");
					setParametersInProgress(false);
					setAllInputsEnabled(true);

				}
				
			};
			sw.execute();
		} else {
			setAllInputsEnabled(true);
		}
	}
	
	private void setParametersInProgress(boolean value) {
		synchronized (this) {
			parametersInProgress = value;
			if (!value) {
				notify();			
			}
		}
	}
	public List<ReportParameter> getCurrentParameters() {
		List<ReportParameter> ps = null;
		synchronized (this) {
			while(parametersInProgress) {
				try { wait(); } catch (Throwable t) {}
			}
			ps = new LinkedList<ReportParameter>(parameters);
		}
		/*debug*/if (ps != null) { for (ReportParameter p: ps) { logger.trace("Current value of parameter '" + (p != null && p.getName() != null ? p.getName() : "<null>") + "' is '" + (p != null && p.getValue() != null && p.getValue().getLabel() != null ? p.getValue().getLabel() : "<null>") + "' ('" + (p != null && p.getValue() != null && p.getValue().getValue() != null && p.getValue().getValue().toString() != null ? p.getValue().getValue().toString() : "<null>") + "')"); } }
		logger.trace("Returning current parameter values.");		
		return ps;
	}
	public Report getReport() {
		logger.trace("Returning current report.");		
		return report;
	}
}
