/* $Id: OrderReport.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.samplereports;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.neodatis.odb.core.query.IQuery;
import org.neodatis.odb.core.query.criteria.Where;
import org.neodatis.odb.impl.core.query.criteria.CriteriaQuery;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Order;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.OrderItem;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.DefaultTextReportPrint;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.InvalidReportParameterException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProcessingException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameterImpl;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameterLabeledValue;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportPrint;

/**
 * @author Ondrej Cerny
 *
 */
public class OrderReport implements Report {
	
	private static final String ORDER_CODE_PARAM_NAME = "orderCode";
	
	ReportDataProvider temporaryDataProvider = null;
	List<ReportParameter> parameters = new LinkedList<ReportParameter>();
	
	public OrderReport() {
		parameters.add(new ReportParameterImpl(
				ORDER_CODE_PARAM_NAME,
				"Order Code", String.class, false, null));
	}
	
	public void addOrder(StringBuilder sb, Order o) {
		
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters()
	 */
	@Override
	public List<ReportParameter> getParameters() {
		return parameters;
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters(java.util.List)
	 */
	@Override
	public List<ReportParameter> getParameters(
			List<ReportParameter> currentValues) {
		// we could also verify whether the order exists :-)
		return currentValues;
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getPrint(java.util.List)
	 */
	@Override
	public ReportPrint getPrint(List<ReportParameter> values)
			throws InvalidReportParameterException, ReportDataProcessingException {
		String orderCode = null; 
		if (values == null) {
			throw new InvalidReportParameterException("Null parameter values.");
		}
		for (ReportParameter p: values) {
			if (ORDER_CODE_PARAM_NAME.equals(p.getName())) {
				ReportParameterLabeledValue v = p.getValue();
				if (v != null && v.getValue() != null && v.getValue() instanceof String) {
					orderCode = (String)v.getValue();
					break;
				}
			}
		}
		if (orderCode == null) {
			throw new InvalidReportParameterException("Null value of " + ORDER_CODE_PARAM_NAME + " not allowed.");
		}
		IQuery q = new CriteriaQuery(Order.class, Where.equal("code", orderCode));
		Collection<Order> orders = temporaryDataProvider.select(q);
		
		StringBuilder b = new StringBuilder();
		b.append("<body><h1>Order Report</h1>");
		if (orders != null && orders.size() > 0) {
			for (Order o: orders) {
				b.append("<table>");
				b.append("<tr>");
				b.append("<td><b>Code:</b></td>");
				b.append("<td>").append(o.getCode() != null ? o.getCode() : "---").append("</td>");
				b.append("<td></td><td></td>");				
				b.append("</tr>");
				
				b.append("<tr>");
				b.append("<td><b>Supplier reg. no.:</b></td>");
				b.append("<td>").append(o.getSupplier() != null && o.getSupplier().getRegNo() != null ? o.getSupplier().getRegNo() : "---").append("</td>");				
				b.append("<td><b>Supplier name:</b></td>");
				b.append("<td>").append(o.getSupplier() != null && o.getSupplier().getName() != null ? o.getSupplier().getName() : "---").append("</td>");
				b.append("</tr>");
				
				b.append("<tr>");				
				b.append("<td><b>Created:</b></td>");
				b.append("<td>").append(o.getCreateDate() != null ? o.getCreateDate().toString() : "---").append("</td>");				
				b.append("<td><b>Closed:</b></td>");
				b.append("<td>").append(o.getCloseDate() != null ? o.getCloseDate().toString() : "---").append("</td>");				
				b.append("</tr>");

				b.append("<tr>");				
				b.append("<td>&nbsp;</td>");
				b.append("<td>&nbsp;</td>");
				b.append("<td>&nbsp;</td>");				
				b.append("<td>&nbsp;</td>");
				b.append("</tr>");
				
				b.append("<tr>");				
				b.append("<td><b>Product</b></td>");
				b.append("<td><b>Id</b></td>");
				b.append("<td><b>Amount</b></td>");				
				b.append("<td></td>");
				b.append("</tr>");
				List<OrderItem> orderItems = o.getOrderItems();
				if (orderItems != null) {
					for(OrderItem i: o.getOrderItems()) {
						if (i == null) {
							continue;
						}
						b.append("<tr>");
						b.append("<td>").append(i.getProduct() != null && i.getProduct().getName() != null ? i.getProduct().getName() : "---").append("</td>");
						b.append("<td>").append(i.getProduct() != null ? Long.toString(i.getProduct().getId()) : "---").append("</td>");
						b.append("<td>").append(Double.toString(i.getAmount())).append("</td>");
						b.append("</tr>");						
					}
					b.append("</table>");						
				} else {
					b.append("</table>");
					b.append("<p>No order items.</p>");
					
				}
			}
		} else {
			b.append("<p>No order found for code '");
			b.append(orderCode);
			b.append("'</p>");
		}
		b.append("<body>");
		
		return new DefaultTextReportPrint("text/html", b.toString());
	}

	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#setDataProvider(org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider)
	 */
	@Override
	public void setDataProvider(ReportDataProvider p) {
		temporaryDataProvider = p;
	}

}
