/* $Id: SalesReport.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.rep.samplereports;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.activity.InvalidActivityException;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.IQuery;
import org.neodatis.odb.core.query.criteria.ICriterion;
import org.neodatis.odb.core.query.criteria.Where;
import org.neodatis.odb.impl.core.query.criteria.CriteriaQuery;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Sale;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SaleItem;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.DefaultTextReportPrint;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.InvalidReportParameterException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProcessingException;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameter;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportParameterImpl;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportPrint;

/**
 * @author Ondrej Cerny
 *
 */
public class SalesReport implements Report {
	private class ProductSaleValues {
		public Product product = null;
		public double amount = 0;
		public double totalPrice = 0;
	}

	private static Logger logger = Logger.getLogger(OverviewOfOrdersReport.class);
	private static final String SALE_DATE_FROM_PARAM_NAME = "from";
	private static final String SALE_DATE_TO_PARAM_NAME = "to";	
	
	ReportDataProvider temporaryDataProvider = null;
	List<ReportParameter> parameters;
	
	public SalesReport() {
		parameters = new LinkedList<ReportParameter>();		
		parameters.add(new ReportParameterImpl(
				SALE_DATE_FROM_PARAM_NAME,
				"From", java.util.Date.class, false, null));
		parameters.add(new ReportParameterImpl(
				SALE_DATE_TO_PARAM_NAME,
				"To", java.util.Date.class, false, null));
	}	
	/**
	  * 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters()
	  */
	@Override
	public List<ReportParameter> getParameters() {
		return parameters;
	}
	/**
	  * 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getParameters(java.util.List)
	  */
	@Override
	public List<ReportParameter> getParameters(
			List<ReportParameter> currentValues) {
		return currentValues;
	}
	/**
	  * 
	  * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#setDataProvider(org.objectweb.dsrg.sofa.demos.sofashop.rep.common.ReportDataProvider)
	  */
	@Override
	public void setDataProvider(ReportDataProvider p) {
		temporaryDataProvider = p;
	}
	
	
	/**
	 * @see org.objectweb.dsrg.sofa.demos.sofashop.rep.common.Report#getPrint(java.util.List)
	 */
	@Override
	public ReportPrint getPrint(List<ReportParameter> values)
			throws InvalidReportParameterException, ReportDataProcessingException {
		java.util.Date from = null;
		java.util.Date to = null;
		try {
			for (ReportParameter p: values) {
				if (SALE_DATE_FROM_PARAM_NAME.equals(p.getName())) {
					from = (java.util.Date)p.getValue().getValue();
				} else if (SALE_DATE_TO_PARAM_NAME.equals(p.getName())) {
					to = (java.util.Date)p.getValue().getValue();
				} else {
					logger.trace("Unknown parameter: " + (p.getName() != null ? p.getName() : "<null-name>"));
				}
			}
		} catch (Throwable t) {
			logger.trace("Error occured when parsing parameters: " + t.getClass().getName() + ": " + t.getMessage());
		} finally {
			if (from == null) {
				throw new InvalidReportParameterException(
						"Wrong value supplied for parameter " +
						SALE_DATE_FROM_PARAM_NAME + ".");
			}
			if (to == null) {
				throw new InvalidReportParameterException(
						"Wrong value supplied for parameter " +
						SALE_DATE_TO_PARAM_NAME + ".");				
			}
			if (!from.before(to)) {
				throw new InvalidReportParameterException(
						"Value of parameter " + SALE_DATE_FROM_PARAM_NAME +
						" must be less than " + SALE_DATE_TO_PARAM_NAME + ".");
			}
		}
		ICriterion criterion = Where.and().
		add(Where.ge("close", from)).
		add(Where.le("close", to));
		
		IQuery qry = new CriteriaQuery(Sale.class, criterion);
		Collection<Sale> sales = temporaryDataProvider.select(qry);
		
		// aggregate sales
		Map<Long, ProductSaleValues> aggregatedValues = new HashMap<Long, ProductSaleValues>();
		if (sales != null) {
			for (Sale s: sales) {
				if (s.getSaleItems() != null) {
					for (SaleItem si: s.getSaleItems()) {
						if (si != null && si.getProduct() != null) {
							ProductSaleValues psv = aggregatedValues.get(Long.valueOf(si.getProduct().getId()));
							if (psv == null) {
								psv = new ProductSaleValues();
								aggregatedValues.put(Long.valueOf(si.getProduct().getId()), psv); 
							}
							psv.product = si.getProduct();
							psv.amount += si.getAmount();
							psv.totalPrice += si.getTotalPrice();						
						}
					}
				}
			}
		}
		
		
		StringBuilder sb = new StringBuilder();
		
		sb.append("<body>");
		sb.append("<h1>Overview of sales</h1>");
		if (!aggregatedValues.isEmpty()) {
			sb.append("<table>");
			
			sb.append("<tr>");
			sb.append("<td><b>Product code</b></td>");
			sb.append("<td><b>Product name</b></td>");
			sb.append("<td><b>Amount</b></td>");
			sb.append("<td><b>Price</b></td>");
			sb.append("</tr>");
			
			for (ProductSaleValues s: aggregatedValues.values()) {
				if (s == null) {
					continue;
				}
				sb.append("<tr>");
				sb.append("<td>").append(s.product != null ? s.product.getId() : "---").append("</td>");
				sb.append("<td>").append(s.product != null && s.product.getName() != null ? s.product.getName() : "---").append("</td>");
				sb.append("<td>").append(s.amount).append("</td>");
				sb.append("<td>").append(s.totalPrice).append("</td>");
				sb.append("</tr>");				
			}
						
			sb.append("</table>");						
		} else {
			sb.append("<p>No sales.</p>");
		}
		sb.append("</body>");		
		return new DefaultTextReportPrint("text/html", sb.toString());
	}

}
