/* $Id: DataFiller.java 810 2009-11-16 16:20:38Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.sample;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Formatter;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;
import org.neodatis.odb.core.query.IQuery;
import org.neodatis.odb.impl.core.query.criteria.CriteriaQuery;
import org.objectweb.dsrg.sofa.SOFAClient;
import org.objectweb.dsrg.sofa.SOFALifecycle;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Store;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Supplier;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;
import org.objectweb.dsrg.sofa.demos.sofashop.db.DatabaseUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.db.ISelect;
import org.objectweb.dsrg.sofa.demos.sofashop.db.ITransact;
import org.objectweb.dsrg.sofa.demos.sofashop.db.TransactionAbortedException;


/** Writes sample data into database.
 * 
 * <p>Writes the data into database just in case the database is empty
 *    - tested is presence of {@link Product}, {@link Store} and 
 *    {@link Supplier}.
 * </p>
 *
 * @author Ondrej Cerny
 *
 */
public class DataFiller implements Runnable, SOFAClient, SOFALifecycle {
	
	private static Logger logger = Logger.getLogger(DataFiller.class);
	
	private static final String REQUIRED_ISELECT_NAME = "sel";
	private static final String REQUIRED_ITRANSACT_NAME = "tx";
	
	/** Transaction provider - required interface connector. */
	//@Required(name = "tx")
	ITransact tx = null;
	
	/** Select provider - required interface connector. */
	//@Required(name = "sel")
	ISelect sel = null;
	
	
	/** Number of suppliers to generate. */
	private int nSuppliers = 10;
	/** Number of stores to generate. */
	private int nStores = 5;
	/** Number of products to generate per store. */
	private int nProductsPerStore = 2;
	/** Random generator. */
	private Random rnd = new Random();
	/** Id counter. */
	private int lastId = 0;
	/** Intermediate storage for generated products. */
	private Collection<Product> productsToStore = null;
	/** Current date. */
	private Calendar now = Calendar.getInstance();
	
	/** Dictionary of product names. */
	private static final String PRODUCT_NAMES[] = {
		"Sofa",
		"Furniture Cream",
		"Chair",
		"Table"
	};
	/** Prices. */
	private static final double FINAL_PRICES[] = {
		1200, 7.5, 259.9, 792 
	};
	/** Dictionary of product descriptions. */
	private static final String PRODUCT_DESCRIPTIONS[] = {
		"Leather sofa",
		"Furniture-cream (price per liter)",
		"Not a very comfortable chair",
	};
	/** Dictionary of names. */
	private static final String PERSON_NAMES[] = {
		"Dale",
		"Jane",
		"James",
		"Emma",
		"Andy",
		"Kylie",
		"Arnold",
		"Kristine"
	};
	/** Dictionary of surnames. */
	private static final String PERSON_SURNAMES[] = {
		"Rascal",
		"Malkovicz",
		"Rimmer",
		"Kochanski",
		"Cooper"
	};
	/** Supplier names suffices. */
	private static final String COMPANY_SUFFICES[] = {
		"and Son",
		"coop."
	};
	
	/** Dictionary of stores. */
	List<Store> stores = null;
	
	/** Dictionary of suppliers. */
	List<Supplier> suppliers = null;
	
	
	/** Worker thread. */
	Thread worker = null;
	
	/** Main method.
	 */
	@Override
	public void run() {
		int attempts = 0;
		int dues = 0;
		while (dues < 10) {
			try {
				while (!doTransaction()) {
					++attempts;
				}
				System.out.printf("SAMPLE DATA FILLER: Finished (after %d attempts).\n", attempts);
				return;
			} catch (DatabaseUnreachableException due) {
				++dues;
				System.out.printf("SAMLE DATA FILLER: Database unreachable. Detailed message: %s\nSAMPLE DATA FILLER: Retrying.\n",
						due.getMessage());
			}
		}
	}
	
	private boolean doTransaction() throws DatabaseUnreachableException {
		try {
			logger.warn("DoTransaction");
			boolean insert = true;
			// check whether to store objects into database...
			logger.warn("Checking Product...");														
			IQuery q = new CriteriaQuery(Product.class);
			Collection<Product> currProducts = tx.fetch(q, 0, 3);
			insert = currProducts == null || currProducts.size() == 0;
			if (insert) {
				logger.warn("Checking Store...");							
				q = new CriteriaQuery(Store.class);
				Collection<Store> currStores = tx.fetch(q, 0, 3);
				insert = currStores == null || currStores.size() == 0;				
			}
			if (insert) {
				logger.warn("Checking Supplier...");											
				q = new CriteriaQuery(Supplier.class);
				Collection<Store> currStores = tx.fetch(q, 0, 3);
				insert = currStores == null || currStores.size() == 0;								
			}
			if (insert) {
				logger.warn("Creating data...");
				Collection<Product> data = getProducts();
				for (Product p : data) {
					tx.store(p);
				}
			} else {
				System.out.printf("SAMPLE DATA FILLER: Database already contains some data, fill skipped.\n");
				logger.warn("DoTransaction-return TRUE");				
				return true;
			}
		} catch (TransactionAbortedException tae) {
		} finally {
			if (tx.isAborted()) {
				tx.rollback();
			} else {
				if (tx.commit()) {
					logger.warn("DoTransaction-return TRUE");									
					return true;
				}
			}
		}
		logger.warn("DoTransaction-return FALSE");						
		return false;
	}
	
	private Collection<Product> getProducts() {
		logger.trace("getProducts.");
		if (productsToStore == null) {
			try {
			productsToStore = new LinkedList<Product>();
			logger.trace("Preparing stores.");
			prepareStoreBase();
			logger.trace("Preparing suppliers.");
			prepareSupplierBase();
			logger.trace("Preparing products.");
			int countProducts = 0;
			for(Store s: stores) {
				for(int i = 0; i < nProductsPerStore; ++i) {
					productsToStore.add(createProduct(s));
					++countProducts;
				}
			}
			logger.trace("Stats.");
			System.out.printf("SAMPE DATA FILLER: Generated %d products among %d stores and %d suppliers.\n",
					countProducts, nStores, nSuppliers);
			} catch (Throwable t) {
				logger.error("Exception " + t.getClass().getName() + ": " + t.getMessage());
				t.printStackTrace();
			}
		}
		return productsToStore;
	}
	private int getNextId() {
		return ++lastId;
	}
	private void prepareSupplierBase() {
		if (suppliers == null) {
			suppliers = new LinkedList<Supplier>();
		}
		for(int i = 0; i < nSuppliers; ++i) {
			Supplier supplier = new Supplier();
			// create name
			String name = PERSON_NAMES[rnd.nextInt(PERSON_NAMES.length)] +
				" " + PERSON_SURNAMES[rnd.nextInt(PERSON_SURNAMES.length)];
			if (rnd.nextBoolean()) {
				name += " " + COMPANY_SUFFICES[rnd.nextInt(COMPANY_SUFFICES.length)];
			}
			supplier.setName(name);
			supplier.setDescription("-no description-");
			supplier.setRegNo("P" + Integer.toString(rnd.nextInt(100)) + "-" + Integer.toOctalString(getNextId()));
			suppliers.add(supplier);
		}
	}
	private void prepareStoreBase() {
		if (stores == null) {
			stores = new LinkedList<Store>();
		}
		for(int i = 0; i < nStores; ++i) {
			Store store = new Store();
			store.setId(i);
			stores.add(store);
		}
	}
	private String getEANFromId(int id) {
		if (id <= 9999) {
			return String.format("8%03d%04d", rnd.nextInt(100),id);
		} else {
			return String.format("8E%06d", id);
		}
	}
	private Product createProduct(Store s) {
		Product p = new Product();
		int id = getNextId();
		p.setId(id);
		p.setDefaultEAN(getEANFromId(id));
		int index = rnd.nextInt(PRODUCT_NAMES.length);
		p.setName(PRODUCT_NAMES[index]);
		p.setDescription(index < PRODUCT_DESCRIPTIONS.length ?
				PRODUCT_DESCRIPTIONS[index] : "-no descriptions-");
		double priceBase = index < FINAL_PRICES.length
			? FINAL_PRICES[index] : FINAL_PRICES[rnd.nextInt(FINAL_PRICES.length)];
		double finalPrice = priceBase + (rnd.nextBoolean()
			? (rnd.nextDouble() * 0.1 * priceBase) : (rnd.nextDouble() * (-0.1) * priceBase));
		p.setDefaultPrice(finalPrice);
		
		LinkedList<SupplyTerms> st = new LinkedList<SupplyTerms>();
		addSupplyTerms(st, s, pickSupplier(), finalPrice);
		while(rnd.nextBoolean()) {
			addSupplyTerms(st, pickStore(), pickSupplier(), finalPrice);
		}
		p.setSupplyTerms(st);
		return p;
	}
	private Supplier pickSupplier() {
		return suppliers.get(rnd.nextInt(suppliers.size()));
	}
	private Store pickStore() {
		return stores.get(rnd.nextInt(stores.size()));		
	}
	private void addSupplyTerms(LinkedList<SupplyTerms> list, Store store, Supplier supplier, double finalPrice) {
		int count = 1 + rnd.nextInt(10);
		int diff = rnd.nextInt(10) * 100;
		int i = 0;
		for (; i < count; ++i) {
			list.add(createSupplyTerms(store, supplier,
					i*diff, (i+1)*diff, (0.90 - 0.07*i) * finalPrice)); 
		}
		list.add(createSupplyTerms(store, supplier,
				i*diff, -1, (0.90 - 0.07*i) * finalPrice)); 
		
	}
	private SupplyTerms createSupplyTerms(Store store, Supplier supplier,
			double lowerLimit, double upperLimit, double price) {
		SupplyTerms st = new SupplyTerms();
		st.setStore(store);
		st.setSupplier(supplier);
		st.setLowerLimitIncl(lowerLimit);
		st.setUpperLimitExcl(upperLimit);
		st.setPrice(price);
		setDates(st);
		return st;
	}
	
	private void setDates(SupplyTerms st) {
		long fromTime = now.getTimeInMillis();
		long toTime = now.getTimeInMillis();
		if (rnd.nextBoolean()) {
			// future
			double diff = rnd.nextDouble() * 0.01;
			fromTime += fromTime * diff;
			toTime += toTime * diff * 10;			
		} else {
			// valid now
			fromTime -= fromTime * rnd.nextDouble() * 0.1;
			toTime += toTime * rnd.nextDouble() * 0.1;
		}
		st.setValidFrom(new Date(fromTime));
		st.setValidTo(new Date(toTime));		
	}	
	
	@Override
	public void setRequired(String arg0, Object arg1) {
		logger.trace("DataFiller:setRequired(" +
				(arg0 != null ? arg0 : "<null>") + ", " +
				(arg1 != null ? arg1.getClass().getName() : "<null>") + ")");
		if (arg1 != null && arg0 != null) {
			if (arg0.equals(REQUIRED_ITRANSACT_NAME) && arg1 instanceof ITransact) {
				tx = (ITransact)arg1;
			} else if (arg1.equals(REQUIRED_ISELECT_NAME) && arg1 instanceof ISelect) {
				sel = (ISelect)arg1;
			} else {
				logger.trace("DataFiller:setReuired(): unknown interface");
			}
		}
	}
	@Override
	public void stopping() {
		logger.trace("DataFiller:Stopping()");
	}
	/** Starts this component.
	 */
	@Override
	public void start() {
		logger.trace("DateFiller:Start()");
		worker = new Thread(this);
		worker.start(); 
	}
	/** Stops this component.
	 */
	@Override
	public void stop() {
		logger.trace("DataFiller:Stop()");
		try {
			worker.join();
		} catch (InterruptedException ie) {
			logger.trace("DataFiller:Interrupted during Join.");
		}
		logger.trace("DataFiller:Stopped()");		
	}	
}
