/* $Id: InventoryResourceManager.java 762 2009-10-12 11:19:00Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.store.client;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.log4j.Logger;

/**
 * @author Ondrej Cerny
 *
 */
public class InventoryResourceManager {
	private static Logger logger = Logger.getLogger(InventoryResourceManager.class);
	private Map<Integer, Lock> resourceStateMap =
		new HashMap<Integer, Lock>();
	
	
	
	public void acquireResource(int resourceId) {
		
		Lock resourceLock = null;
		Integer key = new Integer(resourceId);
		synchronized (this) {
			while (!resourceStateMap.containsKey(key)) {
				try {
					wait();
				} catch (InterruptedException e) {
					logger.debug("Interrupted on wait.");
				}
			}
			resourceLock = resourceStateMap.get(key);
		}
		
		if (resourceLock != null) {
			resourceLock.lock();
		} else {
			logger.fatal("Null resource lock.");
			// throw new NullPointerException("Null resource lock.");
		}
	}
	
	public void releaseResource(int resourceId) {
		Lock resourceLock = null;
		Integer key = new Integer(resourceId);
		
		synchronized (this) {
			if (!resourceStateMap.containsKey(key)) {
				resourceStateMap.put(key, new ReentrantLock());
				notifyAll();
			} else {
				resourceLock = resourceStateMap.get(key);
			}
		}
		
		if (resourceLock != null) {
			resourceLock.unlock();
		}
	}
}
