package org.objectweb.dsrg.sofa.demos.sofashop.store.client;

import javax.swing.SwingUtilities;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.objectweb.dsrg.sofa.SOFAClient;
import org.objectweb.dsrg.sofa.SOFALifecycle;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.order.IOrder;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.prod.IListProducts;
import org.objectweb.dsrg.sofa.demos.sofashop.rep.IReporting;

public class Shell implements SOFAClient, SOFALifecycle {
	
	private static Logger logger = Logger.getLogger(Shell.class);
	private SwingInitializer sInitializer;
	
	/** Constructor - starts the gui.
	  * 
	  */
	public Shell() {
		logger.trace("initialization");
		sInitializer = new SwingInitializer();
	}

	/** @see SOFAClient#setRequired(String, Object)
	  * 
	  */
	@Override
	public void setRequired(String arg0, Object arg1) {
		logger.trace(
				"arg0=" + (arg0 != null ? arg0 : "<null>") + ", " +
				"arg1=" + (arg1 != null ? arg1.getClass().getName() : "<null>") + ".");
		if (arg1 != null && arg0 != null) {
			if (arg1 instanceof IOrder) {
				logger.trace("Setting interface '" + arg0 +"' endpoint.");
				InventoryDataHelper.setOrderRegistry((IOrder) arg1);				
			} else if (arg1 instanceof IListProducts) {
				logger.trace("Setting interface '" + arg0 +"' endpoint.");
				InventoryDataHelper.setProductsList((IListProducts) arg1);
			} else if (arg1 instanceof IReporting) {
				logger.trace("Setting interface '" + arg0 + "' endpoing.");
				InventoryDataHelper.setRSC(new ReportingServiceClient((IReporting) arg1));
			} else {
				logger.warn("Unkown interface '" + arg0 + "'endpoint. Ignoring.");
			}
		} else {
			logger.warn("Null interface name or endpoint. Ignoring.");
		}
	}
	
	/** Debugging purposes.
	  * 
	  */
	public static void main(String args[]) {
		configureLog4j();
		// starts the gui
		Shell shell = new Shell();
		
		// TODO: read some input to assign some dummy IListStock and IOrder to InventoryDataHelper
		shell.setRequired(null, null);
		
		shell.start();
		
	}
	/** Configures Log4j if it has not been configured yet.
	  * 
	  */
	public static void configureLog4j() {
		Logger root = Logger.getRootLogger();
		if (root != null && !root.getAllAppenders().hasMoreElements()) {
			// not null and no appenders => not configured yet
			root.setLevel(Level.ALL);
			root.addAppender(new ConsoleAppender(
					new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN)));
			logger.warn("Log4j configured.");
		} else {
			System.err.printf("Log4j already configured or null root logger.");
		}
	}

	@Override
	public void start() {
		SwingUtilities.invokeLater(sInitializer);
	}

	@Override
	public void stop() {
		logger.trace("stop");
	}

	@Override
	public void stopping() {
		logger.trace("stopping");
		SwingUtilities.invokeLater(sInitializer);
	}
	

}
