/* $Id: MainTabbedPane.java 762 2009-10-12 11:19:00Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui;

import java.awt.Container;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.delivery.OrderDeliveryPane;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.order.OrderItemDialog;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.order.OrderOverviewPane;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.products.ProductsListPane;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.reports.ReportsPane;

/**
  * @author Ondrej Cerny
  *
  */
public class MainTabbedPane extends JTabbedPane {
	
	private static Logger logger = Logger.getLogger(MainTabbedPane.class);
	
	private ProductsListPane productsListPane = null;
	private OrderOverviewPane orderOverviewPane = null;
	private ReportsPane reportsPane = null;
	private OrderDeliveryPane orderDeliveryPane = null;
	
	/** Default constructor.
	  */
	public MainTabbedPane() {
		super();
		init();
	}
	
	private void init() {
		// all tabs listed here:
		addTab("Reports", getReportsPane());
		JPanel pane = new JPanel();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		pane.add(getProductsListPane());
		pane.add(getOrderOverviewPane());
		addTab("Order Products", pane);
		addTab("Accept Delivered Orders", getOrderDeliveryPane());
		
	}
	
	private ProductsListPane getProductsListPane() {
		if (productsListPane == null) {
			productsListPane = new ProductsListPane();
		}
		return productsListPane;
	}
	
	private OrderOverviewPane getOrderOverviewPane() {
		if (orderOverviewPane == null) {
			orderOverviewPane = new OrderOverviewPane(getReportsPane());
		}
		return orderOverviewPane;
	}
	
	private ReportsPane getReportsPane() {
		logger.trace("here");
		if (reportsPane == null) {
			reportsPane = new ReportsPane();
		}
		return reportsPane;
	}
	private OrderDeliveryPane getOrderDeliveryPane() {
		if (orderDeliveryPane == null) {
			orderDeliveryPane = new OrderDeliveryPane(getReportsPane());
		}
		return orderDeliveryPane;
	}
	
	private JFrame getParentFrame() {
		Container parent = null;
		for (	parent = this.getParent();
				(parent != null && (!(parent instanceof JFrame)));
				parent = parent.getParent()) {}
		if (parent != null) {
			return (JFrame) parent; 
		} else {
			logger.error("Unable to get parent frame");
			return null;
		}
	}
	
	public void showOrderItemDialog(Product p) {
		JFrame frame = getParentFrame();
		if (p != null && frame != null) {
			OrderItemDialog oid = new OrderItemDialog(getParentFrame(), p);
			oid.setVisible(true);

			if (oid.isSubmitted()) {
				logger.trace("Order item submitted.");
				try {
					getOrderOverviewPane().addOrderItem(
							oid.getProduct(), oid.getSupplyTerms(), oid.getAmount());
				} catch (IllegalArgumentException iae) {
					JOptionPane.showMessageDialog(
							this, "Could not add order item to order.",
							"Error", JOptionPane.ERROR_MESSAGE);					
				}
			} else {
				logger.trace("Order item cancelled.");
			}
		} else {
			if (p == null) {
				JOptionPane.showMessageDialog(
						this, "Could not obtain product details.",
						"Error", JOptionPane.ERROR_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(
						this, "Internal error. Unable to show order dialog.",
						"Error", JOptionPane.ERROR_MESSAGE);				
			}
		}
	}
}
