/* $Id: OrderDeliveryPane.java 775 2009-10-23 00:49:10Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.delivery;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingWorker;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.order.IOrder;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.InventoryDataHelper;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.reports.ReportsPane;

/**
 * @author ondra
 *
 */
public class OrderDeliveryPane extends JPanel implements ActionListener {
	/** Logger. */
	private static Logger logger = Logger.getLogger(OrderDeliveryPane.class);
	/** Reports pane handler for getting order report. */
	private ReportsPane reports;
	
	/** Creates this pane bound to reports provider.
	  * 
	  * @param reports reports provider
	  */
	public OrderDeliveryPane(ReportsPane reports) {
		super();
		this.reports = reports;
		init();
	}
	
	
	/*### Combined swing and logic part ###*/
	/** Returns user provided order code.
	  */
	private String getOrderCode() {
		String result = getOrderCodeInput().getText();
		return result == null ? null : (result.equals("") ? null : result);
	}
	/** Shows order report for current order code.
	  * 
	  * @returns true if and only if the report has been successfully
	  *          found and shown (also the order has been shown)
	  * @see #getOrderCode()  
	  */
	private void showOrderReport() {
		// disable all controls
		setControlsEnabled(false);
		String oc = getOrderCode();
		final String currentOrderCode = oc != null ? new String(oc) : null;
		
		SwingWorker<Object, Object> sw = new SwingWorker<Object, Object>() {
			private JPanel report = null;
			private String errorMessage = null;
			@Override
			protected Object doInBackground() throws Exception {
				errorMessage = "Invalid order code.";
				String orderCode = currentOrderCode;
				report = null;
				if (orderCode != null) {
					report = reports.getOrderReportPane(orderCode);
					errorMessage = "Order report could not be shown.";
				}
				return null;
			}
			@Override
			public void done() {
				JPanel repPane = getOrderReportPane();
				repPane.removeAll();
				
				if (report != null) {
					repPane.add(report);
					repPane.updateUI();					
				} else {
					JLabel e = new JLabel(
							errorMessage != null ?
									errorMessage :
									"Order report not shown; unkown error.");
					e.setForeground(Color.RED);
					repPane.add(e);
					repPane.updateUI();
				}
				setControlsEnabled(true);
				getAcceptOrderDeliveryButton().setEnabled(report != null);
			}

		};
		sw.execute();
	}
	
	/** Closes (set delivered) order with selected code.
	  *
	  * @see IOrder#receiveOrder(String)
	  * 
	  */
	private void closeOrder() {
		setControlsEnabled(false);
		String oc = getOrderCode();
		if (oc == null || oc.equals("")) {
			JOptionPane.showMessageDialog(this, "Enter order code, please.", "No order code.", JOptionPane.WARNING_MESSAGE);
			return;
		}
		final String orderCode = new String(oc);
		
		SwingWorker<Object, Object> sw = new SwingWorker<Object, Object>() {
			private String errorMessage = null;
			@Override
			protected Object doInBackground() throws Exception {
				try {
					InventoryDataHelper.closeOrder(orderCode);					
				} catch (Throwable t) {
					errorMessage = t.getMessage();
				}
				return null;
			}
			@Override
			public void done() {
				if (errorMessage != null) {
					JOptionPane.showMessageDialog(
							getThisPane(), "Error during closing order: " + errorMessage, "Error", JOptionPane.ERROR_MESSAGE); 
				} else {
					JOptionPane.showMessageDialog(
							getThisPane(), "Order closed successfully.", "Order closed", JOptionPane.INFORMATION_MESSAGE); 					
				}
				setControlsEnabled(true);
			}

		};
		sw.execute();
		
		
	}
	
	
	/*### Pure swing part: ###*/
	private JTextField orderCodeInput = null;
	private JButton showButton = null;
	private JButton acceptOrderDeliveryButton = null;
	private JPanel orderReportPane = null;
	private static final String ACCEPT_ORDER_COMMAND_STRING = "A";
	private static final String SHOW_ORDER_COMMAND_STRING = "S";

	
	private void init() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		JPanel p = null;
		
		p = new JPanel();
		p.setLayout(new GridBagLayout());
		GridBagConstraints c;
		c = new GridBagConstraints();
		c.weightx = 2; c.anchor = GridBagConstraints.EAST;		
		p.add(new JLabel("Order code:"), c);
		c = new GridBagConstraints();
		c.weightx = 1; c.anchor = GridBagConstraints.EAST; c.fill = GridBagConstraints.HORIZONTAL;		
		p.add(getOrderCodeInput(), c);
		c = new GridBagConstraints();
		c.weightx = 0; c.anchor = GridBagConstraints.EAST;
		p.add(getShowButton(), c);
		this.add(p);
		
		this.add(getOrderReportPane());
		
		p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.RIGHT));
		p.add(getAcceptOrderDeliveryButton());
		this.add(p);
		
		showOrderReport();
	}
	
	JPanel getThisPane() {
		return this;
	}
	
	private JTextField getOrderCodeInput() {
		if (orderCodeInput == null) {
			orderCodeInput = new JTextField();
			orderCodeInput.setActionCommand(SHOW_ORDER_COMMAND_STRING);
			orderCodeInput.addActionListener(this);
		}
		return orderCodeInput;
	}
	private JButton getShowButton() {
		if (showButton == null) {
			showButton = new JButton("Show");
			showButton.setActionCommand(SHOW_ORDER_COMMAND_STRING);
			showButton.addActionListener(this);
		}
		return showButton;
	}
	private JButton getAcceptOrderDeliveryButton() {
		if (acceptOrderDeliveryButton == null) {
			acceptOrderDeliveryButton = new JButton("Accept");
			acceptOrderDeliveryButton.setActionCommand(ACCEPT_ORDER_COMMAND_STRING);
			acceptOrderDeliveryButton.addActionListener(this);
			acceptOrderDeliveryButton.setEnabled(false);
		}
		return acceptOrderDeliveryButton;
	}
	private JPanel getOrderReportPane() {
		if (orderReportPane == null) {
			orderReportPane = new JPanel();
			orderReportPane.setLayout(new BoxLayout(orderReportPane, BoxLayout.Y_AXIS));
			orderReportPane.setPreferredSize(new Dimension(10000, 10000));
			orderReportPane.setMinimumSize(new Dimension(300, 300)); 
			
			/*orderReportPane.setBorder(
					BorderFactory.createTitledBorder("Order")); */
		}
		return orderReportPane;
	}
	private void setControlsEnabled(boolean en) {
		getOrderCodeInput().setEnabled(en);
		getOrderReportPane().setEnabled(en);
		getShowButton().setEnabled(en);
		getAcceptOrderDeliveryButton().setEnabled(false); // this ctrl must be enabled explicitly
	}
	@Deprecated
	private GridBagConstraints getReportGridBagConstraint() {
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.CENTER;
		return c;
	}

	
	// +---------------------+
	//  ActionListener method
	// +---------------------+
	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (ACCEPT_ORDER_COMMAND_STRING.equals(command)) {
			logger.warn("Accept order.");
			closeOrder();
			
			
		} else if (SHOW_ORDER_COMMAND_STRING.equals(command)) {
			logger.trace("Show order report.");
			showOrderReport();
		} else {
			logger.error("Unrecognize action command.");
		}
	}
}
