/* $Id: OrderItemDSummaryPane.java 806 2009-11-15 10:08:58Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.order;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.BorderFactory;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;


import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Supplier;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;

/**
  * 
  * @author Ondrej Cerny
  *
  */
public class OrderItemDSummaryPane extends JPanel implements ActionListener {
	
	private static Logger logger = Logger.getLogger(OrderItemDSummaryPane.class);
	
	// constants
	private static final String DEFAULT_PRODUCT_NAME_VAL = "--name--";
	private static final String DEFAULT_PRODUCT_DESC_VAL = "--description--";
	private static final String DEFAULT_PRODUCT_ID_VAL = "--#id#--";
	private static final String DEFAULT_PRODUCT_EAN_VAL = "--EAN--";
	private static final String DEFAULT_SUPPLIER_NAME_VAL = "--name--";
	private static final String DEFAULT_SUPPLIER_ID_VAL = "--#id#--";
	private static final String DEFAULT_ORDERITEM_UNITPRICE_VAL = "--#unitPrice#--";
	private static final String DEFAULT_ORDERITEM_PRICE_VAL = "--#totalPrice#--";
	private static final String DEFAULT_ORDERITEM_AMOUNT_VAL = "--#amount#--";
	private static final String DEFAULT_WARNING_VAL = "";
	private static final String NO_VALUE_STRING = "---";
	private static final int GRIDL_ROWS = 0;
	private static final int GRIDL_COLS = 5;
	
	// swing components
	private JLabel productName = null;
	private JLabel productDesc = null;
	private JLabel productId = null;
	private JLabel productEAN = null;
	private JLabel supplierName = null;
	private JLabel supplierId = null;
	private JLabel orderItemUnitPrice = null;
	private JLabel orderItemPrice = null;
	private JLabel orderItemAmount = null;
	private JLabel warningLabel = null;
	private JPanel productPane = null;
	private JPanel supplierPane = null;
	private JPanel orderItemPane = null;
	private JButton submitButton = null;
	
	private boolean submitted = false;
	
	/** Product.
	  */
	private Product product = null;
	/** Supply terms.
	  */
	private SupplyTerms supplyTerms = null;
	/** Number of ordered items.
	  */
	private double amount = 0;
		
	/** Constructor.
	  */
	public OrderItemDSummaryPane(Product p) {
		this.product = p;
		init();
	}
	
	
	protected void init() {
		//setBorder(
		//		BorderFactory.createTitledBorder("Summary"));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		GridBagConstraints cval = new GridBagConstraints();
		cval.anchor = GridBagConstraints.WEST; cval.weightx = 1;
		GridBagConstraints clab = new GridBagConstraints();
		clab.anchor = GridBagConstraints.WEST; clab.weightx = 0;
		
		
		
		JPanel pane = getProductPane();
		pane.setLayout(new GridBagLayout());
		pane.add(new JLabel("Name:"), clab);
		pane.add(getProductNameLabel(), cval);		
		pane.add(new JLabel("Desc.:"), clab);
		pane.add(getProductDescLabel(), cval);				
		pane.add(new JLabel("Id:"), clab);
		pane.add(getProductIdLabel(), cval);				
		pane.add(new JLabel("Def. EAN:"), clab);
		pane.add(getProductEANLabel(), cval);
		add(pane);
		
		pane = getSupplierPane();
		pane.setLayout(new GridBagLayout());
		pane.add(new JLabel("Name:"), clab);
		pane.add(getSupplierNameLabel(), cval);
		pane.add(new JLabel("Id:"), clab);
		pane.add(getSupplierIdLabel(), cval);
		add(pane);
		
		pane = getOrderItemPane();
		pane.setLayout(new GridBagLayout());
		pane.add(new JLabel("Amount:"), clab);
		pane.add(getOrderItemAmountLabel(), cval);
		pane.add(new JLabel("Unit price:"), clab);
		pane.add(getOrderItemUnitPriceLabel(), cval);
		pane.add(new JLabel("Total:"), clab);
		pane.add(getOrderItemPriceLabel(), cval);
		add(pane);
		
		pane = new JPanel();
		pane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		pane.add(getWarningLabel());
		pane.add(getSubmitButton());
		add(pane);
		
		refreshAllFields();
	}
	
	protected JButton getSubmitButton() {
		if (submitButton == null) {
			submitButton = new JButton("Submit");
			submitButton.setEnabled(false);
			submitButton.addActionListener(this);
		}
		return submitButton;
	}
	
	protected JPanel getProductPane() {
		if (productPane == null) {
			productPane = new JPanel();
			productPane.setBorder(
					BorderFactory.createTitledBorder("Product"));
		}
		return productPane;
	}
	
	protected JPanel getSupplierPane() {
		if (supplierPane == null) {
			supplierPane = new JPanel();
			supplierPane.setBorder(
					BorderFactory.createTitledBorder("Supplier"));
		}
		return supplierPane;
	}
	
	protected JPanel getOrderItemPane() {
		if (orderItemPane == null) {
			orderItemPane = new JPanel();
			orderItemPane.setBorder(
					BorderFactory.createTitledBorder("Amount"));
		}
		return orderItemPane;
	}
		
	protected JLabel getProductNameLabel() {
		if (productName == null) {
			productName = new JLabel(DEFAULT_PRODUCT_NAME_VAL);
			productName.setFont(productName.getFont().deriveFont(Font.BOLD));
		}
		return productName;
	}
	protected JLabel getProductDescLabel() {
		if (productDesc == null) {
			productDesc = new JLabel(DEFAULT_PRODUCT_DESC_VAL);
			productDesc.setFont(productDesc.getFont().deriveFont(Font.BOLD));			
		}
		return productDesc;
	}
	protected JLabel getProductIdLabel() {
		if (productId == null) {
			productId = new JLabel(DEFAULT_PRODUCT_ID_VAL);
			productId.setFont(productId.getFont().deriveFont(Font.BOLD));						
		}
		return productId;
	}
	protected JLabel getProductEANLabel() {
		if (productEAN == null) {
			productEAN = new JLabel("vyser se na to");
			productEAN.setText("vyser si");
			productEAN.setFont(productEAN.getFont().deriveFont(Font.BOLD));									
		}
		return productEAN;
	}
	protected JLabel getSupplierNameLabel() {
		if (supplierName == null) {
			supplierName = new JLabel(DEFAULT_SUPPLIER_NAME_VAL);
			supplierName.setFont(supplierName.getFont().deriveFont(Font.BOLD));												
		}
		return supplierName;
	}
	protected JLabel getSupplierIdLabel() {
		if (supplierId == null) {
			supplierId = new JLabel(DEFAULT_SUPPLIER_ID_VAL);
			supplierId.setFont(supplierId.getFont().deriveFont(Font.BOLD));															
		}
		return supplierId;
	}
	protected JLabel getOrderItemAmountLabel() {
		if (orderItemAmount == null) {
			orderItemAmount = new JLabel(DEFAULT_ORDERITEM_AMOUNT_VAL);
			orderItemAmount.setFont(orderItemAmount.getFont().deriveFont(Font.BOLD));																		
		}
		return orderItemAmount;
	}
	protected JLabel getOrderItemUnitPriceLabel() {
		if (orderItemUnitPrice == null) {
			orderItemUnitPrice = new JLabel(DEFAULT_ORDERITEM_UNITPRICE_VAL);
			orderItemUnitPrice.setFont(orderItemUnitPrice.getFont().deriveFont(Font.BOLD));																					
		}
		return orderItemUnitPrice;	
	}
	protected JLabel getOrderItemPriceLabel() {
		if (orderItemPrice == null) {
			orderItemPrice = new JLabel(DEFAULT_ORDERITEM_PRICE_VAL);
			orderItemPrice.setFont(orderItemPrice.getFont().deriveFont(Font.BOLD));																								
		}
		return orderItemPrice;
	}
	protected JLabel getWarningLabel() {
		if (warningLabel == null) {
			warningLabel = new JLabel(DEFAULT_WARNING_VAL);
		}
		return warningLabel;
	}

	protected void refreshProductFields() {
		String name = NO_VALUE_STRING;
		String desc = NO_VALUE_STRING;
		String id = NO_VALUE_STRING;
		String EAN = NO_VALUE_STRING;
		if (product != null) {
			name = product.getName();
			desc = product.getDescription();
			id = String.format("%5d", product.getId());
			EAN = product.getDefaultEAN();
		}
		
		
		getProductNameLabel().setText(name);
		getProductDescLabel().setText(desc);
		getProductIdLabel().setText(id);
		getProductEANLabel().setText(EAN);

		name = getProductNameLabel().getText();
		desc = getProductDescLabel().getText();
		id = getProductIdLabel().getText();
		EAN = getProductEANLabel().getText();
		
		logger.trace("Refreshed product (" +
				"name=" + (name != null ? name : "<null>") +
				"desc=" + (desc != null ? desc : "<null>") +
				"id=" + (id != null ? id : "<null>") +
				"EAN=" + (EAN != null ? EAN : "<null>") +												
				").");

	}
	protected void refreshSupplierFields() {
		logger.trace("Refreshing supplier fields.");		
		String name = NO_VALUE_STRING;
		String id = NO_VALUE_STRING;
		Supplier supplier = supplyTerms != null
					? supplyTerms.getSupplier() : null;
		if (supplier != null) {
			name = supplier.getName();
			id = supplier.getRegNo();
		}
		
		getSupplierNameLabel().setText(name);
		getSupplierIdLabel().setText(id);
	}
	protected void refreshOrderItemFields() {
		logger.trace("Refreshing order item fields.");		
		String amount = String.format("%6.2f", (this.amount));
		String unitPrice = NO_VALUE_STRING;
		String price = NO_VALUE_STRING;

		if (supplyTerms != null) {
			double unitPriceD = supplyTerms.getPrice();
			unitPrice = String.format("%6.2f", unitPriceD);
			price = String.format("%6.2f", unitPriceD * this.amount);
		}
		
		getOrderItemAmountLabel().setText(amount);
		getOrderItemUnitPriceLabel().setText(unitPrice);
		getOrderItemPriceLabel().setText(price);
	}
		
	/** Refreshes display values.
	  * 
	  */
	public void refreshAllFields() {
		refreshProductFields();
		refreshSupplierFields();
		refreshOrderItemFields();
		getSubmitButton().setEnabled(product != null
					&& supplyTerms != null
					&& amount > 0);
	}
	/** Sets product to be ordered by the current order item.
	  * 
	  * <b>
	  * Btw. refreshes display values (see {@link #refreshAllFields()}
	  * </b>
	  * 
	  * @param product new product
	  */
	public void setProduct(Product product) {
		if (this.product != product) {
			logger.trace("Replacing product and cleaning supply terms selection.");
			this.product = product;
			this.supplyTerms = null;
		}
		refreshAllFields();
	}	
	/** Sets supply terms to be used for the current order item.
	  * 
	  * <b>
	  * Btw. refreshes display values (see {@link #refreshAllFields()}
	  * </b>
	  * 
	  * @param supplyTerms new supply terms
	  */
	protected void setSupplyTerms(SupplyTerms supplyTerms) {
		this.supplyTerms = supplyTerms;
		refreshAllFields();
	}	
	/** Sets amount of product units to be ordered.
	  * 
	  * <b>
	  * Btw. refreshes display values (see {@link #refreshAllFields()}
	  * </b>
	  * 
	  * @param amount amount of product units
	  */
	public void setAmount(double amount) {
		this.amount = amount;
		refreshAllFields();
	}
	
	public double getAmount() {
		return this.amount;
	}
	public Product getProduct() {
		return this.product;
	}
	public SupplyTerms getSupplyTerms() {
		return this.supplyTerms;
	}
	public boolean isSubmitted() {
		return submitted;
	}

	// +---------------------+
	//  ActionListener method
	// +---------------------+
	@Override
	public void actionPerformed(ActionEvent e) {
		// submited
		submitted = true;
		
		// get dialog
		Container parent;
		JDialog dial = null;
		for (	parent = this.getParent();
				parent != null && !(parent instanceof JDialog);
				parent = parent.getParent()) {}
		if (parent != null) {
			dial = (JDialog) parent;
		}
		if (dial != null) {
			// close the dialog
			dial.setVisible(false);
			dial.dispose();
		} else {
			logger.error("Unable to get JDialog.");
		}
	}
}
