/* $Id: OrderItemDSupplyTermsTab.java 806 2009-11-15 10:08:58Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.order;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;

/** 
  * 
  * @author Ondrej Cerny
  *
  */
public class OrderItemDSupplyTermsTab extends JPanel
	implements ChangeListener, CaretListener, ListSelectionListener {
	
	/** Logger */
	private static Logger logger = Logger.getLogger(OrderItemDSupplyTermsTab.class);
	
	private Color warningHidddenColor = new Color(0,0,0,0);
	private Color warningVisibleColor = Color.RED;
	//
	private static final String DEFAULT_WARNING_VALUE = "Row not applicable.";
	
	// swing components
	private JLabel warningLabel = null;
	private JTextField amountInput = null;
	private JCheckBox sortBySupplierChBox = null;
	private JCheckBox filterByAmountChBox = null;
	private JTable supplyTermsTable = null;
	/** Supply terms table model. */
	private OrderItemDSTTableModel supplyTermsTableModel = null;
	/** Last valid value of amount. */
	private double lastAmount = 0;
	/** Last selected row index. */
	private int lastSelectedRowIndex = -1;
	private OrderItemDSummaryPane summary = null;

	/**
	  * 		
	  */
	public OrderItemDSupplyTermsTab(OrderItemDSummaryPane summary) {
		this.summary = summary;
		init();
	}
	
	protected void init() {

		if (summary != null) {
			setProduct(summary.getProduct());
			setAmount(summary.getAmount());
		}

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		// amount
		JPanel amountPane = new JPanel();
		//amountPane.setLayout(new FlowLayout(FlowLayout.LEFT));
		amountPane.setLayout(new GridBagLayout());
		GridBagConstraints c;
		c = new GridBagConstraints();
		c.weightx = 0;
		amountPane.add(new JLabel("Amount:"), c);
		c = new GridBagConstraints();
		c.weightx = 1;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		amountPane.add(getAmountInput(), c);
		c = new GridBagConstraints();
		c.weightx = 1;
		amountPane.add(getWarningLabel(), c);
		add(amountPane);
				
		// swithces
		JPanel switchesPane = new JPanel();
		switchesPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		switchesPane.add(new JLabel("Only applicable:"));
		switchesPane.add(getFilterByAmountChBox());
		switchesPane.add(new JLabel("   "));
		switchesPane.add(new JLabel("Sort by supplier name:"));
		switchesPane.add(getSortBySupplierChBox());
		add(switchesPane);
		
		JScrollPane tableScrollPane = new JScrollPane(getSupplyTermsTable());
		add(tableScrollPane);
	}
	protected JLabel getWarningLabel() {
		if (warningLabel == null) {
			warningLabel = new JLabel(DEFAULT_WARNING_VALUE);
			warningLabel.setForeground(warningHidddenColor);
		}
		return warningLabel;
	}
	
	protected JTextField getAmountInput() {
		if (amountInput == null) {
			amountInput = new JTextField(String.valueOf(lastAmount));
			amountInput.setHorizontalAlignment(JTextField.RIGHT);
			amountInput.setSize(new Dimension(100, amountInput.getSize().height));
			amountInput.addCaretListener(this);
		}
		return amountInput;
	}
	protected JCheckBox getSortBySupplierChBox() {
		if (sortBySupplierChBox == null) {
			sortBySupplierChBox = new JCheckBox();
			sortBySupplierChBox.setSelected(false);
			sortBySupplierChBox.addChangeListener(this);
		}
		return sortBySupplierChBox;
	}
	protected JCheckBox getFilterByAmountChBox() {
		if (filterByAmountChBox == null) {
			filterByAmountChBox = new JCheckBox();
			filterByAmountChBox.setSelected(true);
			filterByAmountChBox.addChangeListener(this);
		}
		return filterByAmountChBox;
	}
	protected JTable getSupplyTermsTable() {
		if (supplyTermsTable == null) {
			supplyTermsTable = new JTable(getSupplyTermsTableModel());
			supplyTermsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			
			//supplyTermsTable.setRowSelectionAllowed(true);
			//supplyTermsTable.setColumnSelectionAllowed(false);
			//supplyTermsTable.setCellSelectionEnabled(false);
			ListSelectionModel lsm = supplyTermsTable.getSelectionModel();
			lsm.addListSelectionListener(this);
		}
		return supplyTermsTable;
	}
	protected OrderItemDSTTableModel getSupplyTermsTableModel() {
		if (supplyTermsTableModel == null) {
			supplyTermsTableModel = new OrderItemDSTTableModel();
			supplyTermsTableModel.setIsFiltered(isTableFiltered());
			supplyTermsTableModel.setSortBySupplier(isTableSortedBySupplierName());
		}
		return supplyTermsTableModel; 
	}
	protected boolean isTableFiltered() {
		return getFilterByAmountChBox().isSelected();
	}
	protected boolean isTableSortedBySupplierName() {
		return getSortBySupplierChBox().isSelected();
	}
	protected OrderItemDSummaryPane getSummaryPane() {
		return summary;
	}
	protected boolean isIndexApplicable(int index) {
		return (index >= 0 && index < getSupplyTermsTable().getRowCount());
	}
	protected void setTableRowSelected(int index, boolean selected) {
	  try {
		JTable tab = getSupplyTermsTable();
		if (isIndexApplicable(index)) {
			if (selected) {
				tab.addRowSelectionInterval(index, index);
			} else {
				tab.removeRowSelectionInterval(index, index);
			}
		}
	  } catch (Throwable t) {
		  logger.warn("Exception: " + t.getMessage());
	  }
	}
	protected void setWarningVisible(boolean visible) {
		getWarningLabel().setForeground(
				visible ? warningVisibleColor : warningHidddenColor);
	}
	
	//protected boolead validate()

	/** Sets the product to get the supply terms from.
	  *
	  * <p>
	  * Delegates the call to the {@link #supplyTermsTableModel}.
	  * In particular, the
	  * {@link OrderItemDSTTableModel#dataValuesOf(Product)}
	  * method is called.
	  * </p> 
	  * 
	  * @param product
	  */
	public void setProduct(Product product) {
		getSupplyTermsTableModel().dataValuesOf(product);
	}
	/** Sets amount (to be used for filtering supply terms).
	  * 
	  * <p>
	  * Updates the text field and delegates the call to the
	  * {@link #supplyTermsTableModel}.
	  * </p>
	  * 
	  * @param amount
	  */
	public void setAmount(double amount) {
		getAmountInput().setText(String.valueOf(amount));
		getSupplyTermsTableModel().setAmount(amount);	
	}
	/** Returns last valid value of amount (input text field).
	  *
	  * @return amount
	  */
	public double getAmount() {
		return lastAmount;
	}
	

	//+---------------------+
	// ChangeListener method
	//+---------------------+
	/**
	  * 
	  */
	@Override
	public void stateChanged(ChangeEvent e) {
		JCheckBox checkBox = null;
		Object srcObj = (e != null ? e.getSource() : null);
		if (srcObj instanceof JCheckBox) {
			checkBox = (JCheckBox)srcObj;
		}
		
		if (checkBox == getSortBySupplierChBox()) {			
			// the decision whether to sort by supplier name has been changed
			getSupplyTermsTableModel().setSortBySupplier(checkBox.isSelected());			
		} else if (checkBox == getFilterByAmountChBox()) {
			// the decision whether to filter inapplicable supply terms out
			// has been changed
			getSupplyTermsTableModel().setIsFiltered(checkBox.isSelected());			
		} else {
			logger.error("Unable to determine source component of " + 
					"this ChangeEvent:" + e.toString());
		}
	}

	//+--------------------+
	// CaretListener method
	//+--------------------+
	/**
	  * 
	  */
	@Override
	public void caretUpdate(CaretEvent e) {
		Object srcObj = e.getSource();
		JTextField textField = null;
		if (srcObj != null && srcObj instanceof JTextField) {
			textField = (JTextField) srcObj;
		}
		
		if (textField != null && textField == getAmountInput()) {
			// the value of amount has been changed
			try {
				double value = Double.valueOf(textField.getText()).doubleValue();
				if (value >= 0) {
					lastAmount = value;
				} else {
					throw new NumberFormatException("Negative amount is invalid.");
				}
				getSupplyTermsTableModel().setAmount(value);
				getSummaryPane().setAmount(lastAmount);
			} catch (Throwable t) {
				logger.warn("Current value of the amountInput is invalid: " +
						t.getMessage());
			}
		} else {
			logger.error("Unable to determine carret event source component.");
		}
	}
	//+----------------------------+
	// ListSelectionListener method
	//+----------------------------+
	/**
	  * 
	  */
	@Override
	public void valueChanged(ListSelectionEvent e) {
		int index = getSupplyTermsTable().getSelectedRow();
		if (index >= 0) {
			logger.trace("Selected row no. " + index + ".");
			SupplyTerms st = getSupplyTermsTableModel().getSupplyTerms(index);
		
		
			OrderItemDSummaryPane summaryPane = getSummaryPane();
			if (OrderItemDSTTableModel.isSupplyTermsApplicableToAmount(
					st, summaryPane.getAmount()) 
					&& OrderItemDSTTableModel.isSupplyTermsApplicableRightNow(st)) {
				summaryPane.setSupplyTerms(st);
				setWarningVisible(false);
				lastSelectedRowIndex = index;				
			} else {
				setWarningVisible(true);				
				if (index != lastSelectedRowIndex &&
						isIndexApplicable(lastSelectedRowIndex)) {
					setTableRowSelected(lastSelectedRowIndex, true);
					
				} else {
					getSummaryPane().setSupplyTerms(null);
					logger.warn("Illegal current selection illegal " +
							"as well as the previous selection.");
				}
			}			
		} else if (isIndexApplicable(lastSelectedRowIndex)) {
			// TODO: fix the reselection with respect to table rows reordering 
			logger.trace("Reselecting row " + lastSelectedRowIndex);
			setTableRowSelected(lastSelectedRowIndex, true);
		} else {
			logger.warn("Row deselected.");
			getSummaryPane().setSupplyTerms(null);
		}
	}
}
