/* $Id: OrderItemDialog.java 762 2009-10-12 11:19:00Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.order;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.SupplyTerms;

/**
  * @author Ondrej Cerny
  */
public class OrderItemDialog extends JDialog {
	
	/** Logger. */
	private static Logger logger = Logger.getLogger(OrderItemDialog.class);
	
	// constants
	private static final int DIALOG_WIDTH = 800;
	private static final int DIALOG_HEIGTH = 600;
	private static final int INNER_PANE_WIDTH = DIALOG_HEIGTH - 20;
	private static final String TITLE_PREFIX = "Order Product ";
		
	// swing components
	private OrderItemDSummaryPane summaryPane = null; // <- JPanel
	private OrderItemDSupplyTermsTab supplyTermsPane = null; // <- JPanel
	private JButton submitButton = null;
	
	/** Constructor.
	  * 
	  * @param owner father component
	  */
	public OrderItemDialog(JFrame owner, Product product) {
		this(owner, product, 0);
	}
	
	/** Constructor.
	  * 
	  * @param owner father component
	  * @param product product to be ordered
	  * @paeam amount amount
	  */
	public OrderItemDialog(JFrame owner, Product product, double amount) {
		super(owner,TITLE_PREFIX, true);
		
		summaryPane = new OrderItemDSummaryPane(product);
		supplyTermsPane = new OrderItemDSupplyTermsTab(summaryPane);

		if (product != null) {
			supplyTermsPane.setProduct(product);
			summaryPane.setProduct(product);
			String pName = product.getName();
			if (pName != null) {
				setTitle(TITLE_PREFIX + pName);
			}
		}
		if (amount != 0) {
			supplyTermsPane.setAmount(amount);
			summaryPane.setAmount(amount);
		}
		
		init();
	}
		
	private void init() {
		logger.trace("Initializing dialog.");
		super.dialogInit();
		this.setSize(DIALOG_WIDTH, DIALOG_HEIGTH);		
		JPanel pane = new JPanel();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		pane.add(getSupplyTermsPane());
		pane.add(getSummaryPane());
		add(pane);		
	}
	
	/** Returns summary pane.
	  * 
	  * @return {@link OrderItemDSummaryPane summary pane}
	  */
	protected OrderItemDSummaryPane getSummaryPane() {
		return summaryPane;
	}
	/** Returns supply terms pane
	  * (selection of amount and supply terms for product).
	  * 
	  * @return {@link OrderItemDSupplyTermsTab supply terms pane} 
	  */
	protected OrderItemDSupplyTermsTab getSupplyTermsPane() {
		return supplyTermsPane;
	}
	
	public Product getProduct() {
		return getSummaryPane().getProduct();
	}
	
	public SupplyTerms getSupplyTerms() {
		return getSummaryPane().getSupplyTerms();
	}
	
	public double getAmount() {
		return getSummaryPane().getAmount();
	}
	public boolean isSubmitted() {
		return getSummaryPane().isSubmitted();
	}
}
