/* $Id: ProductsListPane.java 810 2009-11-16 16:20:38Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.products;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingWorker;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.inv.InventoryUnreachableException;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.InventoryDataHelper;
import org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.MainTabbedPane;

/**
 * @author Ondrej Cerny
 *
 */
public class ProductsListPane extends JPanel implements ListSelectionListener, ActionListener {
	
	private static Logger logger = Logger.getLogger(ProductsListPane.class);
	
	private static final String REFRESH_ICON_RESOURCE_PATH = "/resources/icons/32x32/refresh.png";
	private static final String REFRESH_ICON_LIGHT_RESOURCE_PATH = "/resources/icons/32x32/refresh_light.png";
	private static final String REFRESH_ICON_DARK_RESOURCE_PATH = "/resources/icons/32x32/refresh_dark.png";
	
	private JButton refresh = null;
	private JButton order = null;
	private JTable table = null;
	private ProductsTableModel productsTableModel = null;

	public ProductsListPane() {
		super();
		init();
	}
	private void init() {
		setBorder(BorderFactory.createTitledBorder("Products"));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		JPanel ctrls;
		
		ctrls = new JPanel();
		ctrls.setLayout(new FlowLayout(FlowLayout.RIGHT));
		ctrls.add(getRefresh());
		add(ctrls);
		
		add(new JScrollPane(getTable()));
		
		ctrls = new JPanel();
		ctrls.setLayout(new FlowLayout(FlowLayout.RIGHT));
		ctrls.add(getOrder());
		add(ctrls);
		
		//refreshTableData();		
	}
	private Icon getIcon(String resourcePath) {
		java.net.URL imgUrl = getClass().getResource(resourcePath);
		if (imgUrl != null) {
			ImageIcon ico = new ImageIcon(imgUrl);
			//Image im = ico.getImage();
			return ico;
		} else {
			logger.warn("Unable to find icon resource (" + resourcePath + ").");
			return null;
		}
	}
	private JButton getRefresh() {
		if (refresh == null) {
			Icon icon = getIcon(REFRESH_ICON_RESOURCE_PATH);
			if (icon == null) {
				refresh = new JButton("Refresh");
			} else {
				refresh = new JButton(icon);
				refresh.setContentAreaFilled(false);
				refresh.setRolloverIcon(getIcon(REFRESH_ICON_LIGHT_RESOURCE_PATH));
				refresh.setSelectedIcon(getIcon(REFRESH_ICON_DARK_RESOURCE_PATH));
			}
			refresh.setToolTipText("Refresh the products list.");
			refresh.setBorderPainted(false);
			refresh.addActionListener(this);
		}
		return refresh;
	}
	private JButton getOrder() {
		if (order == null) {
			order = new JButton("Order Product");
			order.setEnabled(false);
			order.addActionListener(this);
		}
		return order;
	}
	private JTable getTable() {
		if (table == null) {
			table = new JTable(getTableModel());
			table.setToolTipText("Select product to order.");
			ListSelectionModel lsm = table.getSelectionModel();
			lsm.addListSelectionListener(this);
		}
		return table;
	}
	private ProductsTableModel getTableModel() {
		if (productsTableModel == null) {
			productsTableModel = new ProductsTableModel();
		}
		return productsTableModel;
	}
	private void setControlsEnabled(boolean b) {
		if (!b || getTableModel().getSelectedRowIndex() >= 0) {
			getOrder().setEnabled(b);
		}
		getRefresh().setEnabled(b);
		getTable().setEnabled(b);
	}

	/** Refreshes table data.
	  *
	  * <p>
	  * Calls {@link InventoryDataHelper#getStockItemsOrderedByProductId(int, int)}
	  * from a {@link SwingWorker}.
	  * </p>
	  * 
	  */
	private void refreshTableData() {
		ProductsTableModel ptm = getTableModel();
		int i = ptm.getSelectedRowIndex();
		if (i >= 0) {
			try {
				getTable().removeRowSelectionInterval(i, i);
			} catch (Throwable thr) {
				logger.warn("Error during deselecting row index.");
			}
		}
		setControlsEnabled(false);
		
		SwingWorker<Object, Object> sw = new SwingWorker<Object, Object> () {
			private Collection<Product> result = null;
			@Override
			protected Object doInBackground() throws Exception {
				try {
					result = InventoryDataHelper.getProducts(null, 0, -1);
				} catch (InventoryUnreachableException iue) {
					logger.warn("InvnetoryUnreachableException: " +iue); 
					// TODO: warn user
				}
				return null;
			}	
			@Override 
			protected void done() {
				getTableModel().setData(result);
				setControlsEnabled(true);
			}
			
		};
		
		sw.execute();
	}
	private MainTabbedPane getMainTabbedPane() {
		Container parent = null;
		for(	parent = this.getParent();
				(parent != null) && (!(parent instanceof MainTabbedPane));
				parent = parent.getParent()) {}
		if (parent != null) {
			return (MainTabbedPane) parent;
		} else {
			logger.error("Unable to get main container.");
			return null;
		}	
	}

	/** Reloads the product and calls parent for showing order dialog.
	  * 
	  * <p>
	  * Calls {@link InventoryDataHelper#getProductForId(long)}
	  * from swing worker for refreshing the field.
	  * </p>
	  * 
	  * @param p
	  */
	private void showOrderDialog(Product p) {
		final long productId = p != null ? p.getId() : -1;
		setControlsEnabled(false);
		if (p != null) {
			SwingWorker<Object, Object> sw = new SwingWorker<Object, Object>() {
				private Product result = null;
				private InventoryUnreachableException iue = null;
				@Override
				protected Object doInBackground() throws Exception {
					try {
						result = InventoryDataHelper.getProductForId(productId);
					} catch (InventoryUnreachableException iue) {
						logger.warn("InventoryUnreachableException: " + iue.getMessage());
						this.iue = iue;
						this.result = null;
					}
					return null;
				}
				@Override
				protected void done() {
					if (result != null) {
						getMainTabbedPane().showOrderItemDialog(result);						
					} else {
						if (iue == null) {
							JOptionPane.showMessageDialog(getMainTabbedPane(),
									"Unable to find the product in inventory. " +
									"Refresh your display, please.", "No Such Product",
									JOptionPane.ERROR_MESSAGE);
						} else {
							JOptionPane.showMessageDialog(getMainTabbedPane(),
									"Invnetory is not reachable. Try again later, please.",
									"Connection Error",	JOptionPane.ERROR_MESSAGE);							
						}
					}
					setControlsEnabled(true);					
				}
			};
			sw.execute();
		} else {
			getMainTabbedPane().showOrderItemDialog(p);
		}
	}
	
	// +----------------------------+
	//  ListSelectionListener method
	// +----------------------------+
	@Override
	public void valueChanged(ListSelectionEvent e) {
		int index = getTable().getSelectedRow();
		getTableModel().setSelectedRowIndex(index);
		getOrder().setEnabled(index >= 0);
	}
	// +---------------------+
	//  ActionListener method  
	// +---------------------+	
	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e != null ? e.getSource() : null;
		if (source == getRefresh()) {
			logger.trace("Refresh button pressed.");
			refreshTableData();
		} else if (source == getOrder()) {
			logger.trace("Order button pressed.");
			showOrderDialog(getTableModel().getSelectedProduct());
		} else {
			logger.error("Unrecoginzed action event source.");
		}
		
	}
}
