/* $Id: ProductsTableModel.java 810 2009-11-16 16:20:38Z ondrac $
 *
 * SofaShop demo for SOFA 2.0 component system
 * Copyright (C) 2009  DSRG, Charles University in Prague
 *                     http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.demos.sofashop.store.client.gui.products;

import java.util.Collection;

import javax.swing.table.AbstractTableModel;

import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.Product;
import org.objectweb.dsrg.sofa.demos.sofashop.common.datamodel.StockItem;

public class ProductsTableModel extends AbstractTableModel {
	
	/** Column names. */
	private String[] columnNames = {"Id", "Name", "Description", "Default EAN"};
	/** Data. */
	private Product[] data = null;
	/** Selected index. */
	private int selectedIndex = -1;
	
	@Override
	public String getColumnName(int index) {
		return columnNames[index];
	}
	@Override
	public int getColumnCount() {
		return columnNames.length;
	}
	@Override
	public int getRowCount() {
		return (data != null) ? data.length : 0;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		
		Product p = data != null && rowIndex >= 0 && rowIndex < data.length
						? data[rowIndex] : null;
		switch(columnIndex) {
			case 0: return p != null ? String.format("%6d", p.getId()) : "---";
			case 1: return p != null ? p.getName() : "---";
			case 2: return p != null ? p.getDescription() : "---";
			case 3: return p != null ? p.getDefaultEAN() : "---";
			default: return null;
		}
	}
	
	/** Sets the data source.
	  * 
	  * @param data list of {@link StockItem stock items}. 
	  */
	public void setData(Collection<Product> data) {
		if (data != null) {
			// note that this might be done with lower memory overhead
			if (this.data == null) {
				this.data = new Product[data.size()];
			}
			this.data = data.toArray(this.data);
			fireTableDataChanged();
		} else {
			if (this.data != null && this.data.length > 0) {
				this.data = null;
			}
		}
	}	
	/** Sets selected index.
	  * 
	  * @param i index
	  */
	public void setSelectedRowIndex(int i) {
		selectedIndex = i;
	}
	/** Returns product related to selected row.
	  * 
	  * @return {@link Product}
	  */
	public Product getSelectedProduct() {
		Product p = (data != null && selectedIndex >= 0 && selectedIndex < data.length)
			? data[selectedIndex] : null;
		return p;
	}
	/** Returns index of selected row.
	  * 
	  * @return index
	  */
	public int getSelectedRowIndex() {
		return selectedIndex; 
	}
}
